@@ -16,7 +16,9 @@ use crate::exit_codes::{CInputError, ExitCode};
 use crate::pointers::{check_ptr, const_ptr_as_ref, mut_ptr_as_ref};
 use num_complex::Complex64;
 
-use qiskit_quantum_info::sparse_observable::{BitTerm, SparseObservable, SparseTermView};
+use qiskit_quantum_info::sparse_observable::{
+    BitTerm, CoherenceError, SparseObservable, SparseTermView,
+};
 
 #[cfg(feature = "python_binding")]
 use pyo3::ffi::PyObject;
@@ -80,8 +82,9 @@ impl TryFrom<&CSparseTerm> for SparseTermView<'_> {
 /// @return A pointer to the created observable.
 ///
 /// # Example
-///
+/// ```c
 ///     QkObs *zero = qk_obs_zero(100);
+/// ```
 ///
 #[no_mangle]
 #[cfg(feature = "cbinding")]
@@ -98,8 +101,9 @@ pub extern "C" fn qk_obs_zero(num_qubits: u32) -> *mut SparseObservable {
 /// @return A pointer to the created observable.
 ///
 /// # Example
-///
+/// ```c
 ///     QkObs *identity = qk_obs_identity(100);
+/// ```
 ///
 #[no_mangle]
 #[cfg(feature = "cbinding")]
@@ -127,26 +131,26 @@ pub extern "C" fn qk_obs_identity(num_qubits: u32) -> *mut SparseObservable {
 ///     to the observable. Otherwise a null pointer is returned.
 ///
 /// # Example
+/// ```c
+///    // define the raw data for the 100-qubit observable |01><01|_{0, 1} - |+-><+-|_{98, 99}
+///    uint32_t num_qubits = 100;
+///    uint64_t num_terms = 2;  // we have 2 terms: |01><01|, -1 * |+-><+-|
+///    uint64_t num_bits = 4; // we have 4 non-identity bits: 0, 1, +, -
+///    QkComplex64 coeffs = {1, -1};
+///    QkBitTerm bits[4] = {QkBitTerm_Zero, QkBitTerm_One, QkBitTerm_Plus, QkBitTerm_Minus};
 ///
-///     // define the raw data for the 100-qubit observable |01><01|_{0, 1} - |+-><+-|_{98, 99}
-///     uint32_t num_qubits = 100;
-///     uint64_t num_terms = 2;  // we have 2 terms: |01><01|, -1 * |+-><+-|
-///     uint64_t num_bits = 4; // we have 4 non-identity bits: 0, 1, +, -
-///
-///     complex double coeffs[2] = {1, -1};
-///     QkBitTerm bits[4] = {QkBitTerm_Zero, QkBitTerm_One, QkBitTerm_Plus, QkBitTerm_Minus};
-///     uint32_t indices[4] = {0, 1, 98, 99};  // <-- e.g. {1, 0, 99, 98} would be invalid
-///     size_t boundaries[3] = {0, 2, 4};
-///
-///     QkObs *obs = qk_obs_new(
-///         num_qubits, num_terms, num_bits, coeffs, bits, indices, boundaries
-///     );
+///    uint32_t indices[4] = {0, 1, 98, 99};  // <-- e.g. {1, 0, 99, 98} would be invalid
+///    size_t boundaries[3] = {0, 2, 4};
+///    QkObs *obs = qk_obs_new(
+///        num_qubits, num_terms, num_bits, &coeffs, bits, indices, boundaries
+///    );
+/// ```
 ///
 /// # Safety
 ///
 /// Behavior is undefined if any of the following conditions are violated:
 ///
-///   * ``coeffs`` is a pointer to a ``complex double`` array of length ``num_terms``
+///   * ``coeffs`` is a pointer to a ``QkComplex64`` array of length ``num_terms``
 ///   * ``bit_terms`` is a pointer to an array of valid ``QkBitTerm`` elements of length ``num_bits``
 ///   * ``indices`` is a pointer to a ``uint32_t`` array of length ``num_bits``, which is
 ///     term-wise sorted in strict ascending order, and every element is smaller than ``num_qubits``
@@ -192,9 +196,10 @@ pub unsafe extern "C" fn qk_obs_new(
 /// @param obs A pointer to the observable to free.
 ///
 /// # Example
-///
+/// ```c
 ///     QkObs *obs = qk_obs_zero(100);
 ///     qk_obs_free(obs);
+/// ```
 ///
 /// # Safety
 ///
@@ -224,16 +229,17 @@ pub unsafe extern "C" fn qk_obs_free(obs: *mut SparseObservable) {
 /// @return An exit code. This is ``>0`` if the term is incoherent or adding the term fails.
 ///
 /// # Example
-///
+/// ```c
 ///     uint32_t num_qubits = 100;
 ///     QkObs *obs = qk_obs_zero(num_qubits);
 ///
-///     complex double coeff = 1;
+///     QkComplex64 coeff = {1, 0};
 ///     QkBitTerm bit_terms[3] = {QkBitTerm_X, QkBitTerm_Y, QkBitTerm_Z};
 ///     uint32_t indices[3] = {0, 1, 2};
-///     QkObsTerm term = {&coeff, 3, bit_terms, indices, num_qubits};
+///     QkObsTerm term = {coeff, 3, bit_terms, indices, num_qubits};
 ///
 ///     int exit_code = qk_obs_add_term(obs, &term);
+/// ```
 ///
 /// # Safety
 ///
@@ -277,12 +283,13 @@ pub unsafe extern "C" fn qk_obs_add_term(
 /// @return An exit code.
 ///
 /// # Example
-///
+/// ```c
 ///     QkObs *obs = qk_obs_identity(100);
 ///     QkObsTerm term;
 ///     int exit_code = qk_obs_term(obs, 0, &term);
 ///     // out-of-bounds indices return an error code
 ///     // int error = qk_obs_term(obs, 12, &term);
+/// ```
 ///
 /// # Safety
 ///
@@ -325,9 +332,10 @@ pub unsafe extern "C" fn qk_obs_term(
 /// @return The number of terms in the observable.
 ///
 /// # Example
-///
+/// ```c
 ///     QkObs *obs = qk_obs_identity(100);
 ///     size_t num_terms = qk_obs_num_terms(obs);  // num_terms==1
+/// ```
 ///
 /// # Safety
 ///
@@ -349,9 +357,10 @@ pub unsafe extern "C" fn qk_obs_num_terms(obs: *const SparseObservable) -> usize
 /// @return The number of qubits the observable is defined on.
 ///
 /// # Example
-///
+/// ```c
 ///     QkObs *obs = qk_obs_identity(100);
 ///     uint32_t num_qubits = qk_obs_num_qubits(obs);  // num_qubits==100
+/// ```
 ///
 /// # Safety
 ///
@@ -373,9 +382,10 @@ pub unsafe extern "C" fn qk_obs_num_qubits(obs: *const SparseObservable) -> u32
 /// @return The number of terms in the observable.
 ///
 /// # Example
-///
+/// ```c
 ///     QkObs *obs = qk_obs_identity(100);
 ///     size_t len = qk_obs_len(obs);  // len==0, as there are no non-trivial bit terms
+/// ```
 ///
 /// # Safety
 ///
@@ -393,21 +403,22 @@ pub unsafe extern "C" fn qk_obs_len(obs: *const SparseObservable) -> usize {
 /// Get a pointer to the coefficients.
 ///
 /// This can be used to read and modify the observable's coefficients. The resulting
-/// pointer is valid to read for ``qk_obs_num_terms(obs)`` elements of ``complex double``.
+/// pointer is valid to read for ``qk_obs_num_terms(obs)`` elements of ``QkComplex64``.
 ///
 /// @param obs A pointer to the observable.
 ///
 /// @return A pointer to the coefficients.
 ///
 /// # Example
+/// ```c
+///    QkObs *obs = qk_obs_identity(100);
+///    size_t num_terms = qk_obs_num_terms(obs);
+///    QkComplex64 *coeffs = qk_obs_coeffs(obs);
 ///
-///     QkObs *obs = qk_obs_identity(100);
-///     size_t num_terms = qk_obs_num_terms(obs);
-///     complex double *coeffs = qk_obs_coeffs(obs);
-///
-///     for (size_t i = 0; i < num_terms; i++) {
-///         printf("%f + i%f\n", creal(coeffs[i]), cimag(coeffs[i]));
-///     }
+///    for (size_t i = 0; i < num_terms; i++) {
+///        printf("%f + i%f\n", coeffs[i].re, coeffs[i].im);
+///    }
+/// ```
 ///
 /// # Safety
 ///
@@ -432,14 +443,14 @@ pub unsafe extern "C" fn qk_obs_coeffs(obs: *mut SparseObservable) -> *mut Compl
 /// @return A pointer to the indices.
 ///
 /// # Example
-///
+/// ```c
 ///     uint32_t num_qubits = 100;
 ///     QkObs *obs = qk_obs_zero(num_qubits);
 ///
-///     complex double coeff = 1;
+///     QkComplex64 coeff = {1, 0};
 ///     QkBitTerm bit_terms[3] = {QkBitTerm_X, QkBitTerm_Y, QkBitTerm_Z};
-///     uint32_t indices[3] = {0, 1, 2};
-///     QkObsTerm term = {&coeff, 3, bit_terms, indices, num_qubits};
+///     uint32_t term_indices[3] = {0, 1, 2};
+///     QkObsTerm term = {coeff, 3, bit_terms, term_indices, num_qubits};
 ///     qk_obs_add_term(obs, &term);
 ///
 ///     size_t len = qk_obs_len(obs);
@@ -448,6 +459,7 @@ pub unsafe extern "C" fn qk_obs_coeffs(obs: *mut SparseObservable) -> *mut Compl
 ///     for (size_t i = 0; i < len; i++) {
 ///         printf("index %i: %i\n", i, indices[i]);
 ///     }
+/// ```
 ///
 ///     qk_obs_free(obs);
 ///
@@ -475,22 +487,23 @@ pub unsafe extern "C" fn qk_obs_indices(obs: *mut SparseObservable) -> *mut u32
 /// @return A pointer to the boundaries.
 ///
 /// # Example
+/// ```c
+///    uint32_t num_qubits = 100;
+///    QkObs *obs = qk_obs_zero(num_qubits);
 ///
-///     uint32_t num_qubits = 100;
-///     QkObs *obs = qk_obs_zero(num_qubits);
+///    QkComplex64 coeff = {1, 0};
+///    QkBitTerm bit_terms[3] = {QkBitTerm_X, QkBitTerm_Y, QkBitTerm_Z};
+///    uint32_t indices[3] = {0, 1, 2};
+///    QkObsTerm term = {coeff, 3, bit_terms, indices, num_qubits};
+///    qk_obs_add_term(obs, &term);
 ///
-///     complex double coeff = 1;
-///     QkBitTerm bit_terms[3] = {QkBitTerm_X, QkBitTerm_Y, QkBitTerm_Z};
-///     uint32_t indices[3] = {0, 1, 2};
-///     QkObsTerm term = {&coeff, 3, bit_terms, indices, num_qubits};
-///     qk_obs_add_term(obs, &term);
+///    size_t num_terms = qk_obs_num_terms(obs);
+///    size_t *boundaries = qk_obs_boundaries(obs);
 ///
-///     size_t num_terms = qk_obs_num_terms(obs);
-///     uint32_t *boundaries = qk_obs_boundaries(obs);
-///
-///     for (size_t i = 0; i < num_terms + 1; i++) {
-///         printf("boundary %i: %i\n", i, boundaries[i]);
-///     }
+///    for (size_t i = 0; i < num_terms + 1; i++) {
+///        printf("boundary %i: %i\n", i, boundaries[i]);
+///    }
+/// ```
 ///
 /// # Safety
 ///
@@ -517,14 +530,14 @@ pub unsafe extern "C" fn qk_obs_boundaries(obs: *mut SparseObservable) -> *mut u
 /// @return A pointer to the bit terms.
 ///
 /// # Example
-///
+/// ```c
 ///     uint32_t num_qubits = 100;
 ///     QkObs *obs = qk_obs_zero(num_qubits);
 ///
-///     complex double coeff = 1;
+///     QkComplex64 coeff = {1, 0};
 ///     QkBitTerm bit_terms[3] = {QkBitTerm_X, QkBitTerm_Y, QkBitTerm_Z};
 ///     uint32_t indices[3] = {0, 1, 2};
-///     QkObsTerm term = {&coeff, 3, bit_terms, indices, num_qubits};
+///     QkObsTerm term = {coeff, 3, bit_terms, indices, num_qubits};
 ///     qk_obs_add_term(obs, &term);
 ///
 ///     size_t len = qk_obs_len(obs);
@@ -535,6 +548,7 @@ pub unsafe extern "C" fn qk_obs_boundaries(obs: *mut SparseObservable) -> *mut u
 ///     }
 ///
 ///     qk_obs_free(obs);
+/// ```
 ///
 /// # Safety
 ///
@@ -556,16 +570,17 @@ pub unsafe extern "C" fn qk_obs_bit_terms(obs: *mut SparseObservable) -> *mut Bi
 /// @param coeff The coefficient to multiply the observable with.
 ///
 /// # Example
-///
+/// ```c
 ///     QkObs *obs = qk_obs_identity(100);
-///     complex double coeff = 2;
+///     QkComplex64 coeff = {2, 0};
 ///     QkObs *result = qk_obs_multiply(obs, &coeff);
+/// ```
 ///
 /// # Safety
 ///
 /// Behavior is undefined if any of the following is violated
 /// * ``obs`` is a valid, non-null pointer to a ``QkObs``
-/// * ``coeff`` is a valid, non-null pointer to a ``complex double``
+/// * ``coeff`` is a valid, non-null pointer to a ``QkComplex64``
 #[no_mangle]
 #[cfg(feature = "cbinding")]
 pub unsafe extern "C" fn qk_obs_multiply(
@@ -589,10 +604,11 @@ pub unsafe extern "C" fn qk_obs_multiply(
 /// @return A pointer to the result ``left + right``.
 ///
 /// # Example
-///
+/// ```c
 ///     QkObs *left = qk_obs_identity(100);
 ///     QkObs *right = qk_obs_zero(100);
 ///     QkObs *result = qk_obs_add(left, right);
+/// ```
 ///
 /// # Safety
 ///
@@ -622,10 +638,11 @@ pub unsafe extern "C" fn qk_obs_add(
 ///     in terms of the matrix multiplication ``@``.
 ///
 /// # Example
-///
+/// ```c
 ///     QkObs *first = qk_obs_zero(100);
 ///     QkObs *second = qk_obs_identity(100);
 ///     QkObs *result = qk_obs_compose(first, second);
+/// ```
 ///
 /// # Safety
 ///
@@ -659,10 +676,11 @@ pub unsafe extern "C" fn qk_obs_compose(
 ///     in terms of the matrix multiplication ``@``.
 ///
 /// # Example
-///
+/// ```c
 ///     QkObs *first = qk_obs_zero(100);
 ///     QkObs *second = qk_obs_identity(100);
 ///     QkObs *result = qk_obs_compose(first, second);
+/// ```
 ///
 /// # Safety
 ///
@@ -702,6 +720,104 @@ pub unsafe extern "C" fn qk_obs_compose_map(
     Box::into_raw(Box::new(result))
 }
 
+/// @ingroup QkObs
+/// Apply a new qubit layout to the observable.
+///
+/// The layout is set by an array ``layout`` of new indices, specifying that qubit at current
+/// index ``i`` is relabelled to index ``layout[i]``. The number of qubits the observable
+/// acts on can be extended by setting a larger ``num_qubits`` than the current observable has.
+///
+/// @param obs A pointer to the observable, this observable will be modified in place upon success.
+/// Check the exit code to ensure the layout was correctly applied.
+/// @param layout A pointer to the layout. The pointer must point to an array to
+/// ``qk_obs_num_qubits(obs)`` elements of type ``uint32_t``. Each element must have values
+/// in ``[0, num_qubits)``.
+/// @param num_qubits The number of output qubits.
+///
+/// @return An exit code.
+/// * ``QkExitCode_Success`` upon success
+/// * ``QkExitCode_DuplicteIndexError`` if duplicate qubit indices were found
+/// * ``QkExitCode_MismatchedQubits`` if ``num_qubits`` is smaller than the number of qubits in
+///   the observable
+/// * ``QkExitCode_IndexError`` for any other index errors, such as invalid values in ``layout``.
+///
+/// # Example
+///
+/// This interface allows to relabel and extend the qubit indices:
+///
+/// ```c
+/// QkObs *obs = qk_obs_zero(4);
+///
+/// // add a term to the observable
+/// QkBitTerm bit_terms[3] = {QkBitTerm_X, QkBitTerm_Y, QkBitTerm_Z};
+/// uint32_t qubits[3] = {1, 2, 3};
+/// complex double coeff = 1;
+/// QkObsTerm term = {coeff, 3, bit_terms, qubits, 4};
+/// qk_obs_add_term(obs, &term);
+///
+/// uint32_t layout[3] = {0, 10, 9};  // qubit mapping is: 0->0, 1->10, 2->9
+/// uint32_t num_output_qubits = 11;
+/// int exit = qk_obs_apply_layout(obs, layout, num_output_qubits);
+/// ```
+///
+/// In a compiler workflow, this function can conveniently be used to apply a
+/// ``QkTranspileLayout*`` obtained from a transpiler pass, called ``transpile_layout``
+/// in the following example:
+///
+/// ```c
+/// // get the number of output qubits
+/// uint32_t num_output_qubits = qk_transpile_layout_num_output_qubits(transpile_layout);
+///
+/// // get the layout including the ancillas (hence the ``false`` in the function call)
+/// uint32_t *layout = malloc(sizeof(uint32_t) * num_output_qubits);
+/// qk_transpile_layout_final_layout(transpile_layout, false, layout);
+///
+/// // apply the layout
+/// int exit = qk_obs_apply_layout(obs, layout, num_output_qubits);
+///
+/// // free the layout array
+/// free(layout);
+/// ```
+///
+/// # Safety
+///
+/// Behavior is undefined if ``obs`` is not a valid, non-null pointer to ``QkObs`` or if ``layout``
+/// is not a valid, non-null pointer to a sequence of ``qk_obs_num_qubits(obs)`` consecutive
+/// elements of ``uint32_t``.
+#[no_mangle]
+#[cfg(feature = "cbinding")]
+pub unsafe extern "C" fn qk_obs_apply_layout(
+    obs: *mut SparseObservable,
+    layout: *const u32,
+    num_qubits: u32,
+) -> ExitCode {
+    // SAFETY: Per documentation, this is a valid, non-null pointer.
+    let obs = unsafe { mut_ptr_as_ref(obs) };
+
+    let layout = if layout.is_null() {
+        None
+    } else {
+        let len = obs.num_qubits() as usize;
+        // SAFETY: Per documentation, ``layout`` is readable for ``obs.num_qubits()`` elements.
+        let indices = unsafe { ::std::slice::from_raw_parts(layout, len) };
+        Some(indices)
+    };
+
+    let obs_with_layout = match obs.apply_layout(layout, num_qubits) {
+        Ok(obs_with_layout) => obs_with_layout,
+        Err(e) => {
+            return match e {
+                CoherenceError::DuplicateIndices => ExitCode::DuplicateIndexError,
+                CoherenceError::NotEnoughQubits { .. } => ExitCode::MismatchedQubits,
+                _ => ExitCode::IndexError,
+            }
+        }
+    };
+
+    *obs = obs_with_layout;
+    ExitCode::Success
+}
+
 /// @ingroup QkObs
 /// Calculate the canonical representation of the observable.
 ///
@@ -711,12 +827,13 @@ pub unsafe extern "C" fn qk_obs_compose_map(
 /// @return The canonical representation of the observable.
 ///
 /// # Example
+/// ```c
+///    QkObs *iden = qk_obs_identity(100);
+///    QkObs *two = qk_obs_add(iden, iden);
 ///
-///     QkObs *iden = qk_obs_identity(100);
-///     QkObs *two = qk_obs_add(iden, iden);
-///
-///     double tol = 1e-6;
-///     QkObs *canonical = qk_obs_canonicalize(two);
+///    double tol = 1e-6;
+///    QkObs *canonical = qk_obs_canonicalize(two, tol);
+/// ```
 ///
 /// # Safety
 ///
@@ -742,9 +859,10 @@ pub unsafe extern "C" fn qk_obs_canonicalize(
 /// @return A pointer to a copy of the observable.
 ///
 /// # Example
-///
+/// ```c
 ///     QkObs *original = qk_obs_identity(100);
 ///     QkObs *copied = qk_obs_copy(original);
+/// ```
 ///
 /// # Safety
 ///
@@ -771,10 +889,11 @@ pub unsafe extern "C" fn qk_obs_copy(obs: *const SparseObservable) -> *mut Spars
 /// @return ``true`` if the observables are equal, ``false`` otherwise.
 ///
 /// # Example
-///
+/// ```c
 ///     QkObs *observable = qk_obs_identity(100);
 ///     QkObs *other = qk_obs_identity(100);
 ///     bool are_equal = qk_obs_equal(observable, other);
+/// ```
 ///
 /// # Safety
 ///
@@ -801,10 +920,11 @@ pub unsafe extern "C" fn qk_obs_equal(
 /// @return A pointer to a nul-terminated char array of the string representation for ``obs``
 ///
 /// # Example
-///
+/// ```c
 ///     QkObs *obs = qk_obs_identity(100);
 ///     char *string = qk_obs_str(obs);
 ///     qk_str_free(string);
+/// ```
 ///
 /// # Safety
 ///
@@ -821,7 +941,7 @@ pub unsafe extern "C" fn qk_obs_equal(
 pub unsafe extern "C" fn qk_obs_str(obs: *const SparseObservable) -> *mut c_char {
     // SAFETY: Per documentation, the pointer is non-null and aligned.
     let obs = unsafe { const_ptr_as_ref(obs) };
-    let string: String = format!("{:?}", obs);
+    let string: String = format!("{obs:?}");
     CString::new(string).unwrap().into_raw()
 }
 
@@ -838,8 +958,15 @@ pub unsafe extern "C" fn qk_obs_str(obs: *const SparseObservable) -> *mut c_char
 #[no_mangle]
 #[cfg(feature = "cbinding")]
 pub unsafe extern "C" fn qk_str_free(string: *mut c_char) {
-    unsafe {
-        let _ = CString::from_raw(string);
+    if !string.is_null() {
+        if !string.is_aligned() {
+            panic!("Attempted to free a non-aligned pointer.")
+        }
+        // SAFETY: Per docstring the pointer is obtained by Qiskit functions, which are
+        // returning strings from CString::into_raw.
+        unsafe {
+            let _ = CString::from_raw(string);
+        }
     }
 }
 
@@ -851,12 +978,13 @@ pub unsafe extern "C" fn qk_str_free(string: *mut c_char) {
 /// @return The function exit code. This is ``>0`` if reading the term failed.
 ///
 /// # Example
-///
+/// ```c
 ///     QkObs *obs = qk_obs_identity(100);
 ///     QkObsTerm term;
 ///     qk_obs_term(obs, 0, &term);
 ///     char *string = qk_obsterm_str(&term);
 ///     qk_str_free(string);
+/// ```
 ///
 /// # Safety
 ///
@@ -874,7 +1002,7 @@ pub unsafe extern "C" fn qk_obsterm_str(term: *const CSparseTerm) -> *mut c_char
     let term = unsafe { const_ptr_as_ref(term) };
 
     let view: SparseTermView = term.try_into().unwrap();
-    let string: String = format!("{:?}", view);
+    let string: String = format!("{view:?}");
     CString::new(string).unwrap().into_raw()
 }
 
@@ -886,10 +1014,11 @@ pub unsafe extern "C" fn qk_obsterm_str(term: *const CSparseTerm) -> *mut c_char
 /// @return The label as ``uint8_t``, which can be cast to ``char`` to obtain the character.
 ///
 /// # Example
-///
+/// ```c
 ///     QkBitTerm bit_term = QkBitTerm_Y;
 ///     // cast the uint8_t to char
 ///     char label = qk_bitterm_label(bit_term);
+/// ```
 ///
 /// # Safety
 ///
