@@ -413,7 +413,7 @@ impl TokenStream {
                     self.done = true;
                     Err(QASM2ParseError::new_err(message_generic(
                         Some(&Position::new(&self.filename, self.line, self.col)),
-                        &format!("lexer failed to read stream: {}", err),
+                        &format!("lexer failed to read stream: {err}"),
                     )))
                 }
             }
@@ -433,7 +433,7 @@ impl TokenStream {
                 self.done = true;
                 Err(QASM2ParseError::new_err(message_generic(
                     Some(&Position::new(&self.filename, self.line, self.col)),
-                    &format!("encountered a non-ASCII byte: {:02X?}", b),
+                    &format!("encountered a non-ASCII byte: {b:02X?}"),
                 )))
             }
             b => Ok(Some(b)),
@@ -452,7 +452,7 @@ impl TokenStream {
                 self.done = true;
                 Err(QASM2ParseError::new_err(message_generic(
                     Some(&Position::new(&self.filename, self.line, self.col)),
-                    &format!("encountered a non-ASCII byte: {:02X?}", b),
+                    &format!("encountered a non-ASCII byte: {b:02X?}"),
                 )))
             }
             b => Ok(Some(b)),
