@@ -16,15 +16,17 @@ use ndarray::ArrayView2;
 use ndarray::{aview2, Array2};
 use num_complex::Complex64;
 use numpy::PyReadonlyArray2;
+use pyo3::exceptions::PyIndexError;
 use pyo3::intern;
 use pyo3::prelude::*;
 use qiskit_circuit::circuit_data::CircuitData;
-use qiskit_circuit::dag_circuit::DAGCircuit;
+use qiskit_circuit::dag_circuit::{DAGCircuit, NodeType};
 use qiskit_circuit::gate_matrix::{
     CH_GATE, CX_GATE, CY_GATE, CZ_GATE, DCX_GATE, ECR_GATE, ISWAP_GATE, ONE_QUBIT_IDENTITY,
     TWO_QUBIT_IDENTITY,
 };
 use qiskit_circuit::imports::{QI_OPERATOR, QUANTUM_CIRCUIT};
+use qiskit_circuit::interner::Interned;
 use qiskit_circuit::operations::StandardGate;
 use qiskit_circuit::operations::{ArrayType, Operation, Param, UnitaryGate};
 use qiskit_circuit::packed_instruction::PackedOperation;
@@ -41,7 +43,6 @@ use qiskit_synthesis::two_qubit_decompose::{
 };
 
 use crate::passes::unitary_synthesis::{PARAM_SET, TWO_QUBIT_BASIS_SET};
-use crate::target::Qargs;
 use crate::target::Target;
 use qiskit_circuit::PhysicalQubit;
 
@@ -138,13 +139,10 @@ fn is_supported(
     target: Option<&Target>,
     basis_gates: Option<&HashSet<String>>,
     name: &str,
-    qargs: &[Qubit],
+    qargs: &[PhysicalQubit],
 ) -> bool {
     match target {
-        Some(target) => {
-            let physical_qargs: Qargs = qargs.iter().map(|bit| PhysicalQubit(bit.0)).collect();
-            target.instruction_supported(name, &physical_qargs)
-        }
+        Some(target) => target.instruction_supported(name, qargs),
         None => match basis_gates {
             Some(basis_gates) => basis_gates.contains(name),
             None => true,
@@ -155,9 +153,30 @@ fn is_supported(
 // If depth > 20, there will be 1q gates to consolidate.
 const MAX_2Q_DEPTH: usize = 20;
 
+struct PhysQargsMap {
+    map: Option<Vec<PhysicalQubit>>,
+    cache: HashMap<Interned<[Qubit]>, Vec<PhysicalQubit>>,
+}
+impl PhysQargsMap {
+    fn new(map: Option<Vec<PhysicalQubit>>) -> Self {
+        Self {
+            map,
+            cache: Default::default(),
+        }
+    }
+    fn get<'a>(&'a mut self, dag: &'a DAGCircuit, key: Interned<[Qubit]>) -> &'a [PhysicalQubit] {
+        let Some(map) = self.map.as_ref() else {
+            return PhysicalQubit::lift_slice(dag.get_qargs(key));
+        };
+        self.cache
+            .entry(key)
+            .or_insert_with(|| dag.get_qargs(key).iter().map(|q| map[q.index()]).collect())
+    }
+}
+
 #[allow(clippy::too_many_arguments)]
 #[pyfunction]
-#[pyo3(name = "consolidate_blocks", signature = (dag, decomposer, basis_gate_name, force_consolidate, target=None, basis_gates=None, blocks=None, runs=None))]
+#[pyo3(name = "consolidate_blocks", signature = (dag, decomposer, basis_gate_name, force_consolidate, target=None, basis_gates=None, blocks=None, runs=None, qubit_map=None))]
 fn py_run_consolidate_blocks(
     dag: &mut DAGCircuit,
     decomposer: DecomposerType,
@@ -167,16 +186,30 @@ fn py_run_consolidate_blocks(
     basis_gates: Option<HashSet<String>>,
     blocks: Option<Vec<Vec<usize>>>,
     runs: Option<Vec<Vec<usize>>>,
+    qubit_map: Option<Vec<PhysicalQubit>>,
 ) -> PyResult<()> {
+    // The node indices that enter from `blocks` and `runs` come from Python space, and we can't
+    // trust that they come from a correct analysis (or the block/run collection might have been
+    // invalidated). Rather than panicking, we should raise Python-space exceptions. We don't have
+    // to check indices that are generated by trusted Rust-only methods within this function.
+    let valid_op_node = |dag: &mut DAGCircuit, index: usize| -> PyResult<NodeIndex> {
+        let index = NodeIndex::new(index);
+        match dag.dag().node_weight(index) {
+            Some(NodeType::Operation(_)) => Ok(index),
+            _ => Err(PyIndexError::new_err(
+                "node index in run or block was not a valid operation",
+            )),
+        }
+    };
     let blocks = match blocks {
         Some(runs) => runs
             .into_iter()
             .map(|run| {
                 run.into_iter()
-                    .map(NodeIndex::new)
-                    .collect::<Vec<NodeIndex>>()
+                    .map(|index| valid_op_node(dag, index))
+                    .collect::<Result<Vec<_>, _>>()
             })
-            .collect(),
+            .collect::<Result<Vec<_>, _>>()?,
         // If runs are specified but blocks are none we're in a legacy configuration where external
         // collection passes are being used. In this case don't collect blocks because it's
         // unexpected.
@@ -186,19 +219,22 @@ fn py_run_consolidate_blocks(
         },
     };
 
-    let runs: Option<Vec<Vec<NodeIndex>>> = runs.map(|runs| {
-        runs.into_iter()
-            .map(|run| {
-                run.into_iter()
-                    .map(NodeIndex::new)
-                    .collect::<Vec<NodeIndex>>()
-            })
-            .collect()
-    });
+    let runs: Option<Vec<Vec<NodeIndex>>> = runs
+        .map(|runs| {
+            runs.into_iter()
+                .map(|run| {
+                    run.into_iter()
+                        .map(|index| valid_op_node(dag, index))
+                        .collect::<Result<Vec<_>, _>>()
+                })
+                .collect::<Result<Vec<_>, _>>()
+        })
+        .transpose()?;
     let mut all_block_gates: HashSet<NodeIndex> =
         HashSet::with_capacity(blocks.iter().map(|x| x.len()).sum());
     // In most cases, the qargs in a block will not exceed 2 qubits.
     let mut block_qargs: HashSet<Qubit> = HashSet::with_capacity(2);
+    let mut phys_qargs = PhysQargsMap::new(qubit_map);
     for block in blocks {
         block_qargs.clear();
         if block.len() == 1 {
@@ -208,7 +244,7 @@ fn py_run_consolidate_blocks(
                 target,
                 basis_gates.as_ref(),
                 inst.op.name(),
-                dag.get_qargs(inst.qubits),
+                phys_qargs.get(dag, inst.qubits),
             ) {
                 all_block_gates.insert(inst_node);
                 let matrix = match get_matrix_from_inst(inst) {
@@ -242,7 +278,7 @@ fn py_run_consolidate_blocks(
                 target,
                 basis_gates.as_ref(),
                 inst.op.name(),
-                dag.get_qargs(inst.qubits),
+                phys_qargs.get(dag, inst.qubits),
             ) {
                 outside_basis = true;
             }
@@ -365,7 +401,7 @@ fn py_run_consolidate_blocks(
             }
             let first_inst_node = run[0];
             let first_inst = dag[first_inst_node].unwrap_operation();
-            let first_qubits = dag.get_qargs(first_inst.qubits);
+            let first_qubits = phys_qargs.get(dag, first_inst.qubits);
 
             if run.len() == 1
                 && !is_supported(
@@ -390,7 +426,6 @@ fn py_run_consolidate_blocks(
                 )?;
                 continue;
             }
-            let qubit = first_qubits[0];
             let mut matrix = ONE_QUBIT_IDENTITY;
 
             let mut already_in_block = false;
@@ -423,8 +458,9 @@ fn py_run_consolidate_blocks(
                 let unitary_gate = UnitaryGate {
                     array: ArrayType::OneQ(array),
                 };
+                let dag_qubit = dag.get_qargs(first_inst.qubits)[0];
                 let mut block_index_map: HashMap<Qubit, usize> = HashMap::with_capacity(1);
-                block_index_map.insert(qubit, 0);
+                block_index_map.insert(dag_qubit, 0);
                 let clbit_pos_map = HashMap::new();
                 dag.replace_block(
                     &run,
@@ -472,6 +508,8 @@ pub fn run_consolidate_blocks(
         None,
         None,
         None,
+        // TODO: this doesn't handle the possibility of control-flow operations yet.
+        None,
     )
 }
 
