@@ -26,7 +26,7 @@ use std::time::Instant;
 use qiskit_circuit::converters::circuit_to_dag;
 use qiskit_circuit::dag_circuit::DAGCircuit;
 use qiskit_circuit::operations::{Operation, OperationRef, Param};
-use qiskit_circuit::rustworkx_core_vnext::isomorphism::vf2;
+use qiskit_circuit::vf2;
 use qiskit_circuit::Qubit;
 
 use super::error_map::ErrorMap;
@@ -166,7 +166,7 @@ fn build_interaction_graph<Ty: EdgeType>(
                     for (outer, inner) in node_qargs.iter().zip(0..inst.op.num_qubits()) {
                         inner_wire_map[inner as usize] = wire_map[outer.index()]
                     }
-                    let block_dag = circuit_to_dag(py, block.extract()?, false, None, None)?;
+                    let block_dag = circuit_to_dag(block.extract()?, false, None, None)?;
                     build_interaction_graph(
                         &block_dag,
                         &inner_wire_map,
@@ -305,7 +305,7 @@ fn build_coupling_map<Ty: EdgeType>(
 
 fn mapping_to_layout<Ty: EdgeType>(
     dag: &DAGCircuit,
-    mapping: IndexMap<usize, usize, ahash::RandomState>,
+    mapping: IndexMap<NodeIndex, NodeIndex, ahash::RandomState>,
     data: &InteractionGraphData<Ty>,
 ) -> HashMap<VirtualQubit, PhysicalQubit> {
     let mut out_layout: HashMap<VirtualQubit, PhysicalQubit> =
@@ -313,8 +313,8 @@ fn mapping_to_layout<Ty: EdgeType>(
 
     for (k, v) in mapping.iter() {
         out_layout.insert(
-            VirtualQubit::new(data.reverse_im_graph_node_map[*v].unwrap().0),
-            PhysicalQubit::new(*k as u32),
+            VirtualQubit::new(data.reverse_im_graph_node_map[k.index()].unwrap().0),
+            PhysicalQubit::new(v.index() as u32),
         );
     }
     out_layout
@@ -325,9 +325,11 @@ fn map_free_qubits(
     mut partial_layout: HashMap<VirtualQubit, PhysicalQubit>,
     reverse_im_graph_node_map: &[Option<Qubit>],
     avg_error_map: &ErrorMap,
+    dag: &DAGCircuit,
     target: &Target,
 ) -> Option<HashMap<VirtualQubit, PhysicalQubit>> {
     if free_nodes.is_empty() {
+        assign_unused_qubits(dag, target, &mut partial_layout);
         return Some(partial_layout);
     }
     let num_physical_qubits = target.num_qubits.unwrap();
@@ -347,10 +349,14 @@ fn map_free_qubits(
             .get(&[PhysicalQubit::new(*qubit_b), PhysicalQubit::new(*qubit_b)])
             .unwrap_or(&0.);
         // Reverse comparison so lower error rates are at the end of the vec.
-        score_b.partial_cmp(&score_a).unwrap()
+        match score_b.partial_cmp(&score_a).unwrap() {
+            Ordering::Equal => qubit_b.cmp(qubit_a),
+            Ordering::Less => Ordering::Less,
+            Ordering::Greater => Ordering::Greater,
+        }
     });
     let mut free_indices: Vec<NodeIndex> = free_nodes.keys().copied().collect();
-    free_indices.par_sort_by_key(|index| free_nodes[index].values().sum::<usize>());
+    free_indices.par_sort_by_key(|index| (free_nodes[index].values().sum::<usize>(), *index));
     for im_index in free_indices {
         let selected_qubit = free_qubits.pop()?;
         partial_layout.insert(
@@ -358,9 +364,34 @@ fn map_free_qubits(
             PhysicalQubit::new(selected_qubit),
         );
     }
+    assign_unused_qubits(dag, target, &mut partial_layout);
     Some(partial_layout)
 }
 
+fn assign_unused_qubits(
+    dag: &DAGCircuit,
+    target: &Target,
+    mapping: &mut HashMap<VirtualQubit, PhysicalQubit>,
+) {
+    if mapping.len() < dag.num_qubits() {
+        let used_qubits = mapping.values().copied().collect::<HashSet<_>>();
+        let mut unused_qubits = (0..target.num_qubits.unwrap()).filter_map(|x| {
+            let phys = PhysicalQubit(x);
+            if !used_qubits.contains(&PhysicalQubit(x)) {
+                Some(phys)
+            } else {
+                None
+            }
+        });
+        for virt in 0..dag.num_qubits() {
+            if mapping.contains_key(&VirtualQubit::new(virt as u32)) {
+                continue;
+            }
+            mapping.insert(VirtualQubit(virt as u32), unused_qubits.next().unwrap());
+        }
+    }
+}
+
 #[pyfunction]
 #[pyo3(signature = (dag, target, strict_direction=false, call_limit=None, time_limit=None, max_trials=None, avg_error_map=None))]
 pub fn vf2_layout_pass(
@@ -380,13 +411,11 @@ pub fn vf2_layout_pass(
         let cm_graph = cm_graph.unwrap();
         let im_graph_data = generate_directed_interaction(dag)?;
         let mappings = vf2::Vf2Algorithm::new(
-            &cm_graph,
             &im_graph_data.im_graph,
-            vf2::NoSemanticMatch,
-            vf2::NoSemanticMatch,
-            false,
-            Ordering::Greater,
+            &cm_graph,
+            (vf2::NoSemanticMatch, vf2::NoSemanticMatch),
             false,
+            vf2::Problem::Subgraph,
             call_limit,
         );
         let max_trials: Option<usize> = match max_trials {
@@ -411,14 +440,17 @@ pub fn vf2_layout_pass(
         let mut chosen_layout_score = f64::MAX;
         let avg_error_map = avg_error_map.unwrap_or_else(|| build_average_error_map(target));
         for mapping in mappings {
+            let (mapping, _score) = mapping.expect("error type is infallible");
             trials += 1;
-            let mapping = mapping_to_layout(dag, mapping.unwrap(), &im_graph_data);
+            let mut mapping = mapping_to_layout(dag, mapping, &im_graph_data);
             if cm_graph.node_count() == im_graph_data.im_graph.node_count() {
+                assign_unused_qubits(dag, target, &mut mapping);
                 return Ok(Some(mapping));
             }
             let layout_score =
                 score_layout_internal(&mapping, &avg_error_map, &im_graph_data, strict_direction)?;
             if layout_score == 0. {
+                assign_unused_qubits(dag, target, &mut mapping);
                 return Ok(Some(mapping));
             }
             if layout_score < chosen_layout_score {
@@ -439,7 +471,10 @@ pub fn vf2_layout_pass(
                 }
             }
         }
-        Ok(chosen_layout)
+        Ok(chosen_layout.map(|mut layout| {
+            assign_unused_qubits(dag, target, &mut layout);
+            layout
+        }))
     } else {
         let cm_graph: Option<StableUnGraph<_, _>> = build_coupling_map(target);
         if cm_graph.is_none() {
@@ -457,6 +492,7 @@ pub fn vf2_layout_pass(
                 HashMap::new(),
                 &im_graph_data.reverse_im_graph_node_map,
                 &avg_error_map,
+                dag,
                 target,
             ));
         }
@@ -477,13 +513,11 @@ pub fn vf2_layout_pass(
             ),
         };
         let mappings = vf2::Vf2Algorithm::new(
-            &cm_graph,
             &im_graph_data.im_graph,
-            vf2::NoSemanticMatch,
-            vf2::NoSemanticMatch,
-            false,
-            Ordering::Greater,
+            &cm_graph,
+            (vf2::NoSemanticMatch, vf2::NoSemanticMatch),
             false,
+            vf2::Problem::Subgraph,
             call_limit,
         );
         let mut trials: usize = 0;
@@ -491,14 +525,17 @@ pub fn vf2_layout_pass(
         let mut chosen_layout: Option<HashMap<VirtualQubit, PhysicalQubit>> = None;
         let mut chosen_layout_score = f64::MAX;
         for mapping in mappings {
+            let (mapping, _score) = mapping.expect("error type is infallible");
             trials += 1;
-            let mapping = mapping_to_layout(dag, mapping.unwrap(), &im_graph_data);
+            let mut mapping = mapping_to_layout(dag, mapping, &im_graph_data);
             if cm_graph.node_count() == im_graph_data.im_graph.node_count() {
+                assign_unused_qubits(dag, target, &mut mapping);
                 return Ok(Some(mapping));
             }
             let layout_score =
                 score_layout_internal(&mapping, &avg_error_map, &im_graph_data, strict_direction)?;
             if layout_score == 0. {
+                assign_unused_qubits(dag, target, &mut mapping);
                 return Ok(Some(mapping));
             }
             if layout_score < chosen_layout_score {
@@ -527,6 +564,7 @@ pub fn vf2_layout_pass(
             chosen_layout,
             &im_graph_data.reverse_im_graph_node_map,
             &avg_error_map,
+            dag,
             target,
         ))
     }
