@@ -32,7 +32,7 @@ author = "Qiskit Development Team"
 # The short X.Y version
 version = "2.2"
 # The full version, including alpha/beta/rc tags
-release = "2.2.2"
+release = "2.2.3"
 
 language = "en"
 
@@ -54,6 +54,9 @@ extensions = [
 
 breathe_projects = {"qiskit": "xml/"}
 breathe_default_project = "qiskit"
+breathe_domain_by_extension = {
+    "h" : "c",
+}
 
 templates_path = ["_templates"]
 
