@@ -279,7 +279,7 @@ class Instruction(Operation):
                 self._params.append(self.validate_parameter(single_param))
 
     def validate_parameter(self, parameter):
-        """Instruction parameters has no validation or normalization."""
+        """Instruction parameter has no validation or normalization."""
         return parameter
 
     def is_parameterized(self):
@@ -446,7 +446,7 @@ class Instruction(Operation):
 
     def __deepcopy__(self, memo=None):
         cpy = copy.copy(self)
-        cpy._params = copy.copy(self._params)
+        cpy._params = copy.deepcopy(self._params, memo)
         if self._definition:
             cpy._definition = copy.deepcopy(self._definition, memo)
         return cpy
