@@ -15,26 +15,20 @@
 from qiskit.circuit.quantumcircuit import QuantumCircuit
 
 
-def clifford_6_5():
+def clifford_2_3():
     """
-    Clifford template 6_5:
+    Clifford template 2_3:
 
     .. code-block:: text
 
-                      ┌───┐
-        q_0: ─■───■───┤ S ├───■───────
-              │ ┌─┴─┐┌┴───┴┐┌─┴─┐┌───┐
-        q_1: ─■─┤ X ├┤ SDG ├┤ X ├┤ S ├
-                └───┘└─────┘└───┘└───┘
+             ┌───┐┌───┐
+        q_0: ┤ H ├┤ H ├
+             └───┘└───┘
 
     Returns:
         QuantumCircuit: template as a quantum circuit.
     """
-    qc = QuantumCircuit(2)
-    qc.cz(0, 1)
-    qc.cx(0, 1)
-    qc.s(0)
-    qc.sdg(1)
-    qc.cx(0, 1)
-    qc.s(1)
+    qc = QuantumCircuit(1)
+    qc.h(0)
+    qc.h(0)
     return qc
