@@ -14,7 +14,6 @@
 
 from __future__ import annotations
 
-import collections
 import contextlib
 import dataclasses
 import io
@@ -63,6 +62,63 @@ from .experimental import ExperimentalFeatures
 from .exceptions import QASM3ExporterError
 from .printer import BasicPrinter
 
+
+@dataclasses.dataclass(frozen=True)
+class DefcalInstruction:
+    """An instruction that should be assumed by the exporter to have an associated ``defcal``
+    statement for it.
+
+    .. note::
+
+        This is not a complete implementation of ``defcal``\\ s in OpenQASM 3, and might require
+        expansion in the future to support a broader set of statements.
+
+        For example:
+
+        .. code-block:: openqasm3
+
+            // We support statements that look like these:
+
+            defcal my_measure q -> bit {}
+            defcal my_reset q {}
+            defcal my_rz(angle phase) q {}
+
+            // ... and _not_ ones that look like these:
+
+            // Uses a non-angle argument.
+            defcal my_multiplied_rz(angle phase, uint amount) q;
+            // Only applies to fixed hardware qubits.
+            defcal my_cx $0, $1 {}
+    """
+
+    name: str
+    """Name of the instruction."""
+    parameters: int
+    """Number of angle-like parameters the instruction takes.
+
+    .. note::
+        This is a partial implementation; ``defcal`` instructions can actually have generic
+        typed parameters in OpenQASM 3, so a more complete representation should allow specifying
+        either angle-like parameters (like ``gate`` has) or other elements that fit into the full type
+        system.
+    """
+    qubits: int
+    """Number of qubits the instruction takes.
+
+    .. note::
+        The typing of this argument is implying that we are only supporting (implicit) ``defcal``
+        statements that are defined over _all_ qubits, whereas in a complete implementation,
+        we would need to account for ``defcal`` statements that only apply to fixed physical qubits.
+    """
+    return_type: types.Type | None
+    """The type of the return value.
+
+    .. note::
+        ``None`` is a stand-in for some type-system representation of a unit or ``void`` type;
+        Qiskit's classical type system does not have (nor need) this at the moment.
+    """
+
+
 # Reserved keywords that gates and variables cannot be named.  It is possible that some of these
 # _could_ be accepted as variable names by OpenQASM 3 parsers, but it's safer for us to just be very
 # conservative.
@@ -139,6 +195,7 @@ class Exporter:
         indent: str = "  ",
         experimental: ExperimentalFeatures = ExperimentalFeatures(0),
         annotation_handlers: dict[str, OpenQASM3Serializer] | None = None,
+        implicit_defcals: dict[str, DefcalInstruction] | None = None,
     ):
         """
         Args:
@@ -179,6 +236,12 @@ class Exporter:
                 :class:`.Annotation` object is encountered, the most specific namespace in this
                 mapping that matches the annotation's :attr:`~.Annotation.namespace` attribute will
                 be used to serialize it.
+            implicit_defcals: mapping of :attr:`.Instruction.name`\\ s to an associated
+                :class:`.DefcalInstruction` object.  All instructions with the key name in the input
+                circuit should be output as if there is a ``defcal`` statement corresponding to the
+                given :class:`.DefcalInstruction` defined.  The key name and the
+                :attr:`.DefcalInstruction.name` do not need to match.  The ``defcal`` name cannot
+                collide with an OpenQASM 3 keyword.
         """
         self.basis_gates = basis_gates
         self.disable_constants = disable_constants
@@ -189,6 +252,7 @@ class Exporter:
         self.indent = indent
         self.experimental = experimental
         self.annotation_handlers = {} if annotation_handlers is None else annotation_handlers
+        self.implicit_defcals = {} if implicit_defcals is None else implicit_defcals
 
     def dumps(self, circuit):
         """Convert the circuit to OpenQASM 3, returning the result as a string."""
@@ -206,6 +270,7 @@ class Exporter:
             allow_aliasing=self.allow_aliasing,
             experimental=self.experimental,
             annotation_handlers=self.annotation_handlers,
+            implicit_defcals=self.implicit_defcals,
         )
         BasicPrinter(stream, indent=self.indent, experimental=self.experimental).visit(
             builder.build_program()
@@ -284,8 +349,8 @@ class GateInfo:
     gate being defined in terms of the `_FIXED_PARAMETERS` objects.  A call-site gate whose
     canonical form equals this can use the corresponding symbol as the callee.
 
-    This can be ``None`` if the gate was an overridden "basis gate" for this export, so no canonical
-    form is known."""
+    This can be ``None`` if the gate was an overridden "basis gate" for this export, in which case
+    there is no associated definition statement of the gate."""
     node: ast.QuantumGateDefinition | None
     """An AST node containing the gate definition.  This can be ``None`` if the gate came from an
     included file, or is an overridden "basis gate" of the export."""
@@ -295,14 +360,16 @@ class SymbolTable:
     """Track Qiskit objects and the OQ3 identifiers used to refer to them."""
 
     def __init__(self):
-        self.gates: collections.OrderedDict[str, GateInfo | None] = {}
+        self.gates: dict[str, GateInfo] = {}
         """Mapping of the symbol name to the "definition source" of the gate, which provides its
-        signature and decomposition.  The definition source can be `None` if the user set the gate
-        as a custom "basis gate".
+        signature and decomposition.
 
         Gates can only be declared in the global scope, so there is just a single look-up for this.
+        This is insertion ordered, and that can be relied on for iteration later.
 
-        This is insertion ordered, and that can be relied on for iteration later."""
+        Note that "gates" defined by a :class:`.DefcalInstruction` might logically correspond to
+        arbitrary non-unitary instructions like ``measure``.
+        """
         self.standard_gate_idents: dict[StandardGate, ast.Identifier] = {}
         """Mapping of standard gate enumeration values to the identifier we represent that as."""
         self.user_gate_idents: dict[int, ast.Identifier] = {}
@@ -498,6 +565,12 @@ class SymbolTable:
             self.user_gate_idents[id(source)] = ident
         return ident
 
+    def register_defcal(self, defcal: DefcalInstruction):
+        """Register a ``defcal`` statement in the symbol table."""
+        # This is a slight hack (things like ``my_measure`` wouldn't be a gate), but should track
+        # the symbols all correctly.
+        self.register_gate_without_definition(defcal.name, None)
+
     def get_gate(self, gate: Gate) -> ast.Identifier | None:
         """Lookup the identifier for a given `Gate`, if it exists."""
         canonical = _gate_canonical_form(gate)
@@ -573,6 +646,7 @@ class QASM3Builder:
         allow_aliasing,
         experimental=ExperimentalFeatures(0),
         annotation_handlers=None,
+        implicit_defcals=None,
     ):
         self.scope = BuildScope(
             quantumcircuit,
@@ -593,6 +667,7 @@ class QASM3Builder:
         self.basis_gates = basis_gates
         self.experimental = experimental
         self.annotation_handlers = {} if annotation_handlers is None else annotation_handlers
+        self.implicit_defcals = {} if implicit_defcals is None else implicit_defcals
 
     @contextlib.contextmanager
     def new_scope(self, circuit: QuantumCircuit, qubits: Iterable[Qubit], clbits: Iterable[Clbit]):
@@ -659,6 +734,17 @@ class QASM3Builder:
 
         for builtin, gate in _BUILTIN_GATES.items():
             self.symbols.register_gate_without_definition(builtin, gate)
+        # The only defcal return types we can handle.  Qiskit doesn't have a way of specifying an
+        # instruction that acts on qubits and returns a ``bit[n]``-like type, so we forbid that
+        # early here too.
+        allowed_defcal_types = (None, types.Bool())
+        for name, defcal in self.implicit_defcals.items():
+            if defcal.return_type not in allowed_defcal_types:
+                raise QASM3ExporterError(
+                    f"The defcal '{defcal.name}' associated with instructions with name '{name}'"
+                    f" returns an unsupported classical type: {defcal.return_type}"
+                )
+            self.symbols.register_defcal(defcal)
         for builtin in self.basis_gates:
             if builtin in _BUILTIN_GATES:
                 # It's built into the langauge; we don't need to re-add it.
@@ -992,8 +1078,10 @@ class QASM3Builder:
                 else:
                     raise RuntimeError(f"unhandled control-flow construct: {instruction.operation}")
                 continue
+            if (defcal := self.implicit_defcals.get(instruction.name, None)) is not None:
+                nodes = [self.build_defcal_call(instruction, defcal)]
             # Build the node, ignoring any condition.
-            if isinstance(instruction.operation, Gate):
+            elif isinstance(instruction.operation, Gate):
                 nodes = [self.build_gate_call(instruction)]
             elif isinstance(instruction.operation, Barrier):
                 operands = [self._lookup_bit(operand) for operand in instruction.qubits]
@@ -1177,7 +1265,7 @@ class QASM3Builder:
         if unit == "expr":
             return self.build_expression(duration)
         if unit == "ps":
-            return ast.DurationLiteral(1000 * duration, ast.DurationUnit.NANOSECOND)
+            return ast.DurationLiteral(duration / 1000, ast.DurationUnit.NANOSECOND)
         unit_map = {
             "ns": ast.DurationUnit.NANOSECOND,
             "us": ast.DurationUnit.MICROSECOND,
@@ -1214,11 +1302,54 @@ class QASM3Builder:
             }
         )
 
+    def _build_quantum_call_parameters(self, parameters) -> list[ast.Expression]:
+        parameters = [
+            ast.StringifyAndPray(self._rebind_scoped_parameters(param)) for param in parameters
+        ]
+        if not self.disable_constants:
+            for parameter in parameters:
+                parameter.obj = pi_check(parameter.obj, output="qasm")
+        return parameters
+
+    def build_defcal_call(self, instruction: CircuitInstruction, defcal: DefcalInstruction):
+        """Build a statement associated with a defcal instruction."""
+        # We forbade return types other than `Bool` and `None` in the initialiser,
+        # but we error so that this breaks if we support the full defcal spec later on.
+        allowed_defcal_types = (None, types.Bool())
+        if defcal.return_type not in allowed_defcal_types:
+            raise QASM3ExporterError(
+                f"The defcal '{defcal.name}' associated with instructions with name '{instruction.name}'"
+                f" returns an unsupported classical type: {defcal.return_type}"
+            )
+        returns_bit = defcal.return_type is not None
+        # Check simple consistency.
+        if (
+            len(instruction.qubits) != defcal.qubits
+            or len(instruction.params) != defcal.parameters
+            # Here, we're assuming that the `clbit` is only an output, not an input.
+            # Qiskit's data model can't actually tell us this, so we're assuming that the
+            # user is correct if they tell us this via the `DefcalInstruction`.
+            or len(instruction.clbits) != int(returns_bit)
+        ):
+            raise QASM3ExporterError(
+                f"Instruction '{instruction!r}' has a call signature that is inconsistent with"
+                f" its associated defcal: '{defcal!r}'."
+            )
+        return ast.DefcalCallStatement(
+            ident=ast.Identifier(defcal.name),
+            parameters=self._build_quantum_call_parameters(instruction.params),
+            qubits=[self._lookup_bit(qubit) for qubit in instruction.qubits],
+            lvalue=self._lookup_bit(instruction.clbits[0]) if returns_bit else None,
+        )
+
     def build_gate_call(self, instruction: CircuitInstruction):
         """Builds a gate-call AST node.
 
         This will also push the gate into the symbol table (if required), including recursively
-        defining the gate blocks."""
+        defining the gate blocks.
+
+        If the operation identifier is found in the symbol table, symbol-resolution will be skipped.
+        """
         operation = instruction.operation
         if hasattr(operation, "_qasm_decomposition"):
             operation = operation._qasm_decomposition()
@@ -1226,13 +1357,7 @@ class QASM3Builder:
         if ident is None:
             ident = self.define_gate(operation)
         qubits = [self._lookup_bit(qubit) for qubit in instruction.qubits]
-        parameters = [
-            ast.StringifyAndPray(self._rebind_scoped_parameters(param))
-            for param in operation.params
-        ]
-        if not self.disable_constants:
-            for parameter in parameters:
-                parameter.obj = pi_check(parameter.obj, output="qasm")
+        parameters = self._build_quantum_call_parameters(instruction.params)
         return ast.QuantumGateCall(ident, qubits, parameters=parameters)
 
 
@@ -1336,6 +1461,9 @@ class _ExprBuilder(expr.ExprVisitor[ast.Expression]):
             unit = node.value.unit()
             if unit == "dt":
                 return ast.DurationLiteral(node.value.value(), ast.DurationUnit.SAMPLE)
+            if unit == "ps":
+                # QASM doesn't natively support picoseconds.
+                return ast.DurationLiteral(node.value.value() / 1000, ast.DurationUnit.NANOSECOND)
             if unit == "ns":
                 return ast.DurationLiteral(node.value.value(), ast.DurationUnit.NANOSECOND)
             if unit == "us":
