@@ -13,6 +13,7 @@
 """Module containing multi-controlled circuits synthesis"""
 
 from .mcmt_vchain import synth_mcmt_vchain
+from .mcmt_xgate import synth_mcmt_xgate
 from .mcx_synthesis import (
     synth_mcx_1_clean_kg24,
     synth_mcx_1_dirty_kg24,
@@ -23,6 +24,7 @@ from .mcx_synthesis import (
     synth_mcx_1_clean_b95,
     synth_mcx_gray_code,
     synth_mcx_noaux_v24,
+    synth_mcx_noaux_hp24,
     synth_c3x,
     synth_c4x,
 )
