@@ -14,17 +14,13 @@
 """
 
 import copy
-import dataclasses
 import logging
-import functools
 import time
 
 import numpy as np
-import rustworkx as rx
 
 from qiskit.converters import dag_to_circuit
 from qiskit.circuit import QuantumRegister
-from qiskit.dagcircuit import DAGCircuit
 from qiskit.transpiler.passes.layout.set_layout import SetLayout
 from qiskit.transpiler.passes.layout.full_ancilla_allocation import FullAncillaAllocation
 from qiskit.transpiler.passes.layout.enlarge_with_ancilla import EnlargeWithAncilla
@@ -33,12 +29,8 @@ from qiskit.transpiler.passmanager import PassManager
 from qiskit.transpiler.layout import Layout
 from qiskit.transpiler.basepasses import TransformationPass
 from qiskit.transpiler.exceptions import TranspilerError
-from qiskit._accelerate import disjoint_utils
-from qiskit._accelerate.nlayout import NLayout
-from qiskit._accelerate.sabre import sabre_layout_and_routing, Heuristic, NeighborTable, SetScaling
-from qiskit.transpiler.passes.routing.sabre_swap import _build_sabre_dag, _apply_sabre_result
-from qiskit.transpiler.target import Target
-from qiskit.transpiler.coupling import CouplingMap
+from qiskit.transpiler.target import Target, _FakeTarget
+from qiskit._accelerate.sabre import sabre_layout_and_routing, Heuristic, SetScaling
 from qiskit.utils import default_num_processes
 
 logger = logging.getLogger(__name__)
@@ -162,37 +154,40 @@ class SabreLayout(TransformationPass):
             both ``routing_pass`` and ``layout_trials`` are specified
         """
         super().__init__()
-        if isinstance(coupling_map, Target):
+        if isinstance(coupling_map, Target) and not isinstance(coupling_map, _FakeTarget):
             self.target = coupling_map
-            self.coupling_map = self.target.build_coupling_map()
+            self._coupling_map = None
         else:
-            self.target = None
-            self.coupling_map = coupling_map
-        self._neighbor_table = None
+            coupling_map = (
+                coupling_map.build_coupling_map()
+                if isinstance(coupling_map, _FakeTarget)
+                # We assume we can mutate this to make it symmetric.
+                else copy.deepcopy(coupling_map)
+            )
+            # A dummy target purely to represent a homogeneous coupling graph for layout purposes.
+            self.target = Target.from_configuration(
+                basis_gates=["u", "cx"], coupling_map=coupling_map
+            )
+            self._coupling_map = coupling_map
+            if self._coupling_map is not None:
+                self._coupling_map.make_symmetric()
         if routing_pass is not None and (swap_trials is not None or layout_trials is not None):
             raise TranspilerError("Both routing_pass and swap_trials can't be set at the same time")
         self.routing_pass = routing_pass
         self.seed = seed
         self.max_iterations = max_iterations
-        self.trials = swap_trials
-        if swap_trials is None:
-            self.swap_trials = default_num_processes()
-        else:
-            self.swap_trials = swap_trials
-        if layout_trials is None:
-            self.layout_trials = default_num_processes()
-        else:
-            self.layout_trials = layout_trials
+        self.swap_trials = default_num_processes() if swap_trials is None else swap_trials
+        self.layout_trials = default_num_processes() if layout_trials is None else layout_trials
         self.skip_routing = skip_routing
-        if self.coupling_map is not None:
-            if not self.coupling_map.is_symmetric:
-                # deepcopy is needed here if we don't own the coupling map (i.e. we were passed it
-                # directly) to avoid modifications updating shared references in passes which
-                # require directional constraints
-                if isinstance(coupling_map, CouplingMap):
-                    self.coupling_map = copy.deepcopy(self.coupling_map)
-                self.coupling_map.make_symmetric()
-            self._neighbor_table = NeighborTable(rx.adjacency_matrix(self.coupling_map.graph))
+
+    @property
+    def coupling_map(self):  # pylint: disable=missing-function-docstring
+        # This property is not intended to be public API, it just keeps backwards compatibility.
+        if self._coupling_map is None:
+            self._coupling_map = self.target.build_coupling_map()
+            if self._coupling_map is not None:
+                self._coupling_map.make_symmetric()
+        return self._coupling_map
 
     def run(self, dag):
         """Run the SabreLayout pass on `dag`.
@@ -205,12 +200,16 @@ class SabreLayout(TransformationPass):
             (otherwise the input dag is returned unmodified).
 
         Raises:
-            TranspilerError: if dag wider than self.coupling_map
+            TranspilerError: if dag wider than the target.
         """
-        if len(dag.qubits) > self.coupling_map.size():
-            raise TranspilerError("More virtual qubits exist than physical.")
+        if self.target.num_qubits is None:
+            raise TranspilerError("given 'Target' was not initialized with a qubit count")
+        if len(dag.qubits) > self.target.num_qubits:
+            raise TranspilerError(
+                f"More virtual qubits ({len(dag.qubits)}) exist"
+                f" than physical ({self.target.num_qubits})."
+            )
 
-        # Choose a random initial_layout.
         if self.routing_pass is not None:
             if not self.coupling_map.is_connected():
                 raise TranspilerError(
@@ -243,238 +242,91 @@ class SabreLayout(TransformationPass):
                     )
                     initial_layout = final_layout
                     circ, rev_circ = rev_circ, circ
-
-                # Diagnostics
-                logger.info("new initial layout")
-                logger.info(initial_layout)
+                logger.info("new initial layout: %s", initial_layout)
 
             for qreg in dag.qregs.values():
                 initial_layout.add_register(qreg)
             self.property_set["layout"] = initial_layout
             self.routing_pass.fake_run = False
             return dag
+
         # Combined
-        if self.target is not None:
-            # This is a special case SABRE only works with a bidirectional coupling graph
-            # which we explicitly can't create from the target. So do this manually here
-            # to avoid altering the shared state with the unfiltered indices.
-            target = self.target.build_coupling_map(filter_idle_qubits=True)
-            target.make_symmetric()
-        else:
-            target = self.coupling_map
-        inner_run = self._inner_run
-        if "sabre_starting_layouts" in self.property_set:
-            inner_run = functools.partial(
-                self._inner_run, starting_layouts=self.property_set["sabre_starting_layouts"]
-            )
-        if self.target is not None:
-            components = disjoint_utils.run_pass_over_connected_components(
-                dag, self.target, inner_run
-            )
-            # If components is None we can't build a coupling map from the target so we must have
-            # one provided:
-            if components is None:
-                temp_target = Target.from_configuration(
-                    basis_gates=["u", "cx"], coupling_map=target
-                )
-                components = disjoint_utils.run_pass_over_connected_components(
-                    dag, temp_target, inner_run
-                )
-        else:
-            temp_target = Target.from_configuration(basis_gates=["u", "cx"], coupling_map=target)
-            components = disjoint_utils.run_pass_over_connected_components(
-                dag, temp_target, inner_run
-            )
-        self.property_set["layout"] = Layout(
-            {
-                component.dag.qubits[logic]: component.coupling_map.graph[phys]
-                for component in components
-                for logic, phys in component.initial_layout.layout_mapping()
-                # A physical component of the coupling map might be wider than the DAG that we're
-                # laying out onto it.  We shouldn't include these implicit ancillas right now as the
-                # ancilla-allocation pass will run on the whole map in one go.
-                if logic < len(component.dag.qubits)
-            }
-        )
+        dag_indices = {qubit: i for i, qubit in enumerate(dag.qubits)}
 
-        # Add the existing registers to the layout
-        for qreg in dag.qregs.values():
-            self.property_set["layout"].add_register(qreg)
+        def to_partial_layout(layout):
+            out = [None] * dag.num_qubits()
+            for virt, phys in layout.get_virtual_bits().items():
+                if (pos := dag_indices.get(virt, None)) is not None:
+                    out[pos] = phys
+            return out
 
-        # If skip_routing is set then return the layout in the property set
-        # and throwaway the extra work we did to compute the swap map.
-        # We also skip routing here if there is more than one connected
-        # component we ran layout on. We can only reliably route the full dag
-        # in this case if there is any dependency between the components
-        # (typically shared classical data or barriers).
-        if self.skip_routing or len(components) > 1:
-            return dag
-
-        # At this point, we become a transformation pass, and apply the layout and the routing to
-        # the DAG directly.  This includes filling in the `property_set` data of the embed passes.
-
-        dag = self._ancilla_allocation_no_pass_manager(dag)
-        # The ancilla-allocation pass has expanded this since we set it above.
-        full_initial_layout = self.property_set["layout"]
-
-        # Set up a physical DAG to apply the Sabre result onto.  We do not need to run the
-        # `ApplyLayout` transpiler pass (which usually does this step), because we're about to apply
-        # the layout and routing together as part of resolving the Sabre result.
-        physical_qubits = QuantumRegister(self.coupling_map.size(), "q")
-        mapped_dag = DAGCircuit()
-        mapped_dag.name = dag.name
-        mapped_dag.metadata = dag.metadata
-        mapped_dag.global_phase = dag.global_phase
-        mapped_dag.add_qreg(physical_qubits)
-        mapped_dag.add_clbits(dag.clbits)
-        for creg in dag.cregs.values():
-            mapped_dag.add_creg(creg)
-        for var in dag.iter_input_vars():
-            mapped_dag.add_input_var(var)
-        for var in dag.iter_captured_vars():
-            mapped_dag.add_captured_var(var)
-        for var in dag.iter_declared_vars():
-            mapped_dag.add_declared_var(var)
-        for stretch in dag.iter_captured_stretches():
-            mapped_dag.add_captured_stretch(stretch)
-        for stretch in dag.iter_declared_stretches():
-            mapped_dag.add_declared_stretch(stretch)
-        self.property_set["original_qubit_indices"] = {
-            bit: index for index, bit in enumerate(dag.qubits)
-        }
-        final_layout = Layout(
-            {
-                mapped_dag.qubits[
-                    component.coupling_map.graph[initial]
-                ]: component.coupling_map.graph[final]
-                for component in components
-                for initial, final in enumerate(component.final_permutation)
-            }
-        )
-
-        # The coupling map may have been split into more components than the DAG.  In this case,
-        # there will be some physical qubits unaccounted for in our `final_layout`.  Strictly the
-        # `if` check is unnecessary, but we can avoid the loop for most circuits and backends.
-        if len(final_layout) != len(physical_qubits):
-            used_qubits = {
-                qubit for component in components for qubit in component.coupling_map.graph.nodes()
-            }
-            for index, qubit in enumerate(physical_qubits):
-                if index in used_qubits:
-                    continue
-                final_layout[qubit] = index
-
-        if self.property_set["final_layout"] is None:
-            self.property_set["final_layout"] = final_layout
-        else:
-            self.property_set["final_layout"] = final_layout.compose(
-                self.property_set["final_layout"], dag.qubits
-            )
-        for component in components:
-            # Sabre routing still returns all its swaps as on virtual qubits, so we need to expand
-            # each component DAG with the virtual ancillas that were allocated to it, so the layout
-            # application can succeed.  This is the last thing we do with the component DAG, so it's
-            # ok for us to modify it.
-            component_size = component.coupling_map.size()
-            dag_size = component.dag.num_qubits()
-            if component_size > dag_size:
-                used_physical = {full_initial_layout[logic] for logic in component.dag.qubits}
-                component.dag.add_qubits(
-                    [
-                        full_initial_layout[component.coupling_map.graph[phys]]
-                        for phys in range(component.dag.num_qubits(), component_size)
-                        if component.coupling_map.graph[phys] not in used_physical
-                    ]
-                )
-            mapped_dag = _apply_sabre_result(
-                mapped_dag,
-                component.dag,
-                component.sabre_result,
-                component.initial_layout,
-                [
-                    mapped_dag.qubits[component.coupling_map.graph[phys]]
-                    for phys in range(component_size)
-                ],
-                component.circuit_to_dag_dict,
-            )
-        disjoint_utils.combine_barriers(mapped_dag, retain_uuid=False)
-        return mapped_dag
-
-    def _inner_run(self, dag, coupling_map, starting_layouts=None):
-        if not coupling_map.is_symmetric:
-            # deepcopy is needed here to avoid modifications updating
-            # shared references in passes which require directional
-            # constraints
-            coupling_map = copy.deepcopy(coupling_map)
-            coupling_map.make_symmetric()
-        neighbor_table = NeighborTable(rx.adjacency_matrix(coupling_map.graph))
-        dist_matrix = coupling_map.distance_matrix
-        original_qubit_indices = {bit: index for index, bit in enumerate(dag.qubits)}
-        partial_layouts = []
-        if starting_layouts is not None:
-            coupling_map_reverse_mapping = {
-                coupling_map.graph[x]: x for x in coupling_map.graph.node_indices()
-            }
-            for layout in starting_layouts:
-                virtual_bits = layout.get_virtual_bits()
-                out_layout = [None] * len(dag.qubits)
-                for bit, phys in virtual_bits.items():
-                    pos = original_qubit_indices.get(bit, None)
-                    if pos is None:
-                        continue
-                    out_layout[pos] = coupling_map_reverse_mapping[phys]
-                partial_layouts.append(out_layout)
-
-        sabre_dag, circuit_to_dag_dict = _build_sabre_dag(
-            dag,
-            coupling_map.size(),
-            original_qubit_indices,
-        )
-        # In our defaults, the basic heuristic shouldn't scale by size; if it does, it's liable to
-        # get the algorithm stuck.  See https://github.com/Qiskit/qiskit/pull/14458 for more.
+        starting_layouts = [
+            to_partial_layout(layout)
+            for layout in self.property_set.get("sabre_starting_layouts", [])
+        ]
         heuristic = (
-            Heuristic(attempt_limit=10 * coupling_map.size())
+            Heuristic(attempt_limit=10 * self.target.num_qubits)
             .with_basic(1.0, SetScaling.Constant)
             .with_lookahead(0.5, 20, SetScaling.Size)
             .with_decay(0.001, 5)
         )
         sabre_start = time.perf_counter()
-        (initial_layout, final_permutation, sabre_result) = sabre_layout_and_routing(
-            sabre_dag,
-            neighbor_table,
-            dist_matrix,
+        # If `skip_routing`, then `out_dag` and `final` are meaningless but well-typed.
+        out_dag, initial, final = sabre_layout_and_routing(
+            dag,
+            self.target,
             heuristic,
-            self.max_iterations,
-            self.swap_trials,
-            self.layout_trials,
-            self.seed,
-            partial_layouts,
+            max_iterations=self.max_iterations,
+            num_swap_trials=self.swap_trials or 1,
+            num_random_trials=self.layout_trials,
+            seed=self.seed,
+            partial_layouts=starting_layouts,
+            skip_routing=self.skip_routing,
         )
         sabre_stop = time.perf_counter()
         logger.debug(
-            "Sabre layout algorithm execution for a connected component complete in: %s sec.",
+            "Sabre layout algorithm execution for all components complete in: %s sec.",
             sabre_stop - sabre_start,
         )
-        return _DisjointComponent(
-            dag,
-            coupling_map,
-            initial_layout,
-            final_permutation,
-            sabre_result,
-            circuit_to_dag_dict,
-        )
 
-    def _ancilla_allocation_no_pass_manager(self, dag):
-        """Run the ancilla-allocation and -enlargement passes on the DAG chained onto our
-        ``property_set``, skipping the DAG-to-circuit conversion cost of using a ``PassManager``."""
-        ancilla_pass = FullAncillaAllocation(self.coupling_map)
-        ancilla_pass.property_set = self.property_set
-        dag = ancilla_pass.run(dag)
-        enlarge_pass = EnlargeWithAncilla()
-        enlarge_pass.property_set = ancilla_pass.property_set
-        dag = enlarge_pass.run(dag)
-        self.property_set = enlarge_pass.property_set
-        return dag
+        if self.skip_routing:
+            virtuals = list(dag.qubits)
+            # In `skip_routing` mode, subsequent passes don't expect ancilla expansion.
+            initial_layout = Layout(
+                {p: virtuals[v] for v, p in initial.layout_mapping() if v < len(virtuals)}
+            )
+            for register in dag.qregs.values():
+                initial_layout.add_register(register)
+            self.property_set["layout"] = initial_layout
+            self.property_set["original_qubit_indices"] = {
+                qubit: i for i, qubit in enumerate(virtuals)
+            }
+            return dag
+
+        ancillas = QuantumRegister(self.target.num_qubits - dag.num_qubits(), "ancilla")
+        virtuals = list(dag.qubits) + list(ancillas)
+        initial_layout = Layout({p: virtuals[v] for v, p in initial.layout_mapping()})
+        for register in dag.qregs.values():
+            initial_layout.add_register(register)
+        if ancillas:
+            initial_layout.add_register(ancillas)
+        self.property_set["layout"] = initial_layout
+        self.property_set["original_qubit_indices"] = {qubit: i for i, qubit in enumerate(virtuals)}
+
+        final_layout = Layout(
+            {out_dag.qubits[initial.virtual_to_physical(v)]: p for v, p in final.layout_mapping()}
+        )
+        if (prev_final_layout := self.property_set.get("final_layout", None)) is None:
+            self.property_set["final_layout"] = final_layout
+        else:
+            # The "final layout" can be thought of as a "comes from" permutation that you apply at
+            # the end of the circuit to invert the routing.  So if there's an existing one, what we
+            # apply at the end of the circuit needs to set the circuit qubits so they "come from"
+            # the previous one, then those "come from" the one we've just added.
+            self.property_set["final_layout"] = prev_final_layout.compose(
+                final_layout, out_dag.qubits
+            )
+        return out_dag
 
     def _layout_and_route_passmanager(self, initial_layout):
         """Return a passmanager for a full layout and routing.
@@ -504,22 +356,3 @@ class SabreLayout(TransformationPass):
         qubit_map = Layout.combine_into_edge_map(initial_layout, trivial_layout)
         final_layout = {v: pass_final_layout._v2p[qubit_map[v]] for v in initial_layout._v2p}
         return Layout(final_layout)
-
-
-@dataclasses.dataclass
-class _DisjointComponent:
-    __slots__ = (
-        "dag",
-        "coupling_map",
-        "initial_layout",
-        "final_permutation",
-        "sabre_result",
-        "circuit_to_dag_dict",
-    )
-
-    dag: DAGCircuit
-    coupling_map: CouplingMap
-    initial_layout: NLayout
-    final_permutation: "list[int]"
-    sabre_result: "tuple[SwapMap, Sequence[int], NodeBlockResults]"
-    circuit_to_dag_dict: "dict[int, DAGCircuit]"
