@@ -183,7 +183,12 @@ not sufficient, the corresponding synthesis method will return `None`.
       - :class:`~.MCXSynthesisNoAuxV24`
       - `0`
       - `0`
-      - quadratic number of CX gates; use instead of ``"gray_code"`` for large values of `k`
+      - quadratic number of CX gates
+    * - ``"noaux_hp24"``
+      - :class:`~.MCXSynthesisNoAuxHP24`
+      - `0`
+      - `0`
+      - linear number of CX gates; use instead of ``"noaux_v24"`` or ``"gray_code"`` for `k>5`
     * - ``"n_clean_m15"``
       - :class:`~.MCXSynthesisNCleanM15`
       - `k-2`
@@ -198,22 +203,22 @@ not sufficient, the corresponding synthesis method will return `None`.
       - :class:`~.MCXSynthesis2CleanKG24`
       - `2`
       - `0`
-      - at most `12*k-18` CX gates
+      - at most `6*k-6` CX gates
     * - ``"2_dirty_kg24"``
       - :class:`~.MCXSynthesis2DirtyKG24`
       - `0`
       - `2`
-      - at most `24*k-48` CX gates
+      - at most `12*k-18` CX gates
     * - ``"1_clean_kg24"``
       - :class:`~.MCXSynthesis1CleanKG24`
       - `1`
       - `0`
-      - at most `12*k-18` CX gates
+      - at most `6*k-6` CX gates
     * - ``"1_dirty_kg24"``
       - :class:`~.MCXSynthesis1DirtyKG24`
       - `0`
       - `1`
-      - at most `24*k-48` CX gates
+      - at most `12*k-18` CX gates
     * - ``"1_clean_b95"``
       - :class:`~.MCXSynthesis1CleanB95`
       - `1`
@@ -230,6 +235,7 @@ not sufficient, the corresponding synthesis method will return `None`.
 
    MCXSynthesisGrayCode
    MCXSynthesisNoAuxV24
+   MCXSynthesisNoAuxHP24
    MCXSynthesisNCleanM15
    MCXSynthesisNDirtyI15
    MCXSynthesis2CleanKG24
@@ -261,6 +267,11 @@ MCMT Synthesis
       - `0`
       - `0`
       - uses Qiskit's standard control mechanism
+    * - ``"xgate"``
+      - :class:`.MCMTSynthesisXGate`
+      - `0`
+      - `0`
+      - uses a linear number of Toffoli gates
     * - ``"default"``
       - :class:`~.MCMTSynthesisDefault`
       - any
@@ -272,6 +283,7 @@ MCMT Synthesis
 
    MCMTSynthesisVChain
    MCMTSynthesisNoAux
+   MCMTSynthesisXGate
    MCMTSynthesisDefault
 
    
@@ -364,6 +376,10 @@ Modular Adder Synthesis
       - Plugin class
       - Number of clean ancillas
       - Description
+    * - ``"modular_v17"``
+      - :class:`.ModularAdderSynthesisV17`
+      - 0
+      - a modular adder without any ancillary qubits
     * - ``"ripple_cdkm"``
       - :class:`.ModularAdderSynthesisC04`
       - 1
@@ -384,6 +400,7 @@ Modular Adder Synthesis
 .. autosummary::
    :toctree: ../stubs/
 
+   ModularAdderSynthesisV17
    ModularAdderSynthesisC04
    ModularAdderSynthesisD00
    ModularAdderSynthesisV95
@@ -478,12 +495,17 @@ Multiplier Synthesis
       - :class:`.MultiplierSynthesisR17`
       - 0
       - a QFT-based multiplier
+    * - ``"default"``
+      - :class:`~.MultiplierSynthesisDefault`
+      - any
+      - chooses the best algorithm based on the ancillas available
 
 .. autosummary::
    :toctree: ../stubs/
 
    MultiplierSynthesisH18
    MultiplierSynthesisR17
+   MultiplierSynthesisDefault
 
 """
 
@@ -498,6 +520,7 @@ from qiskit.circuit.operation import Operation
 from qiskit.circuit.library import (
     LinearFunction,
     QFTGate,
+    XGate,
     MCXGate,
     C3XGate,
     C4XGate,
@@ -559,7 +582,9 @@ from qiskit.synthesis.multi_controlled import (
     synth_mcx_1_clean_b95,
     synth_mcx_gray_code,
     synth_mcx_noaux_v24,
+    synth_mcx_noaux_hp24,
     synth_mcmt_vchain,
+    synth_mcmt_xgate,
 )
 from qiskit.synthesis.evolution import ProductFormula, synth_pauli_network_rustiq
 from qiskit.synthesis.arithmetic import (
@@ -567,6 +592,7 @@ from qiskit.synthesis.arithmetic import (
     adder_qft_d00,
     adder_ripple_v95,
     adder_ripple_r25,
+    adder_modular_v17,
     multiplier_qft_r17,
     multiplier_cumulative_h18,
 )
@@ -574,6 +600,7 @@ from qiskit.quantum_info.operators import Clifford
 from qiskit.transpiler.passes.routing.algorithms import ApproximateTokenSwapper
 from qiskit.transpiler.exceptions import TranspilerError
 from qiskit.circuit._add_control import EFFICIENTLY_CONTROLLED_GATES
+from qiskit.transpiler.optimization_metric import OptimizationMetric
 
 from qiskit._accelerate.high_level_synthesis import synthesize_operation, HighLevelSynthesisData
 from .plugin import HighLevelSynthesisPlugin
@@ -1189,8 +1216,8 @@ class MCXSynthesis2CleanKG24(HighLevelSynthesisPlugin):
     object to use this method with :class:`~.HighLevelSynthesis`.
 
     For a multi-controlled X gate with :math:`k\ge 3` control qubits this synthesis method requires
-    :math:`2` additional clean ancillary qubits. The synthesized circuit consists of :math:`k + 2`
-    qubits and at most :math:`12 * k - 18` CX gates.
+    :math:`2` additional clean ancillary qubits. The synthesized circuit consists of :math:`k + 3`
+    qubits and at most :math:`6 * k - 6` CX gates.
 
     The plugin supports the following plugin-specific options:
 
@@ -1231,8 +1258,8 @@ class MCXSynthesis2DirtyKG24(HighLevelSynthesisPlugin):
     object to use this method with :class:`~.HighLevelSynthesis`.
 
     For a multi-controlled X gate with :math:`k\ge 3` control qubits this synthesis method requires
-    :math:`2` additional dirty ancillary qubits. The synthesized circuit consists of :math:`k + 2`
-    qubits and at most :math:`24 * k - 48` CX gates.
+    :math:`2` additional dirty ancillary qubits. The synthesized circuit consists of :math:`k + 3`
+    qubits and at most :math:`12 * k - 18` CX gates.
 
     The plugin supports the following plugin-specific options:
 
@@ -1276,7 +1303,7 @@ class MCXSynthesis1CleanKG24(HighLevelSynthesisPlugin):
 
     For a multi-controlled X gate with :math:`k\ge 3` control qubits this synthesis method requires
     :math:`1` additional clean ancillary qubit. The synthesized circuit consists of :math:`k + 2`
-    qubits and at most :math:`12 * k - 18` CX gates.
+    qubits and at most :math:`6 * k - 6` CX gates.
 
     The plugin supports the following plugin-specific options:
 
@@ -1318,7 +1345,7 @@ class MCXSynthesis1DirtyKG24(HighLevelSynthesisPlugin):
 
     For a multi-controlled X gate with :math:`k\ge 3` control qubits this synthesis method requires
     :math:`1` additional dirty ancillary qubit. The synthesized circuit consists of :math:`k + 2`
-    qubits and at most :math:`24 * k - 48` CX gates.
+    qubits and at most :math:`12 * k - 18` CX gates.
 
     The plugin supports the following plugin-specific options:
 
@@ -1392,7 +1419,8 @@ class MCXSynthesisNoAuxV24(HighLevelSynthesisPlugin):
 
     For a multi-controlled X gate with :math:`k` control qubits this synthesis
     method requires no additional clean auxiliary qubits. The synthesized
-    circuit consists of :math:`k + 1` qubits.
+    circuit consists of :math:`k + 1` qubits. The number of CX-gates is quadratic in
+    :math:`k`.
 
     References:
         1. Vale et. al., *Circuit Decomposition of Multicontrolled Special Unitary
@@ -1415,11 +1443,24 @@ class MCXSynthesisNoAuxV24(HighLevelSynthesisPlugin):
         return decomposition
 
 
-class MCXSynthesisDefault(HighLevelSynthesisPlugin):
-    r"""The default synthesis plugin for a multi-controlled X gate.
+class MCXSynthesisNoAuxHP24(HighLevelSynthesisPlugin):
+    r"""Synthesis plugin for a multi-controlled X gate based on the
+    paper by Huang and Palsberg.
 
-    This plugin name is :``mcx.default`` which can be used as the key on
+    See [1] for details.
+
+    This plugin name is :``mcx.noaux_hp24`` which can be used as the key on
     an :class:`~.HLSConfig` object to use this method with :class:`~.HighLevelSynthesis`.
+
+    For a multi-controlled X gate with :math:`k` control qubits this synthesis
+    method requires no additional clean auxiliary qubits. The synthesized
+    circuit consists of :math:`k + 1` qubits. The number of CX-gates is linear in
+    :math:`k`.
+
+    References:
+        1. Huang and Palsberg, *Compiling Conditional Quantum Gates without Using
+           Helper Qubits*, PLDI (2024),
+           <https://dl.acm.org/doi/10.1145/3656436>`_
     """
 
     def run(self, high_level_object, coupling_map=None, target=None, qubits=None, **options):
@@ -1432,16 +1473,95 @@ class MCXSynthesisDefault(HighLevelSynthesisPlugin):
             # their definition as it should.
             return None
 
-        # Iteratively run other synthesis methods available
+        num_ctrl_qubits = high_level_object.num_ctrl_qubits
+        decomposition = synth_mcx_noaux_hp24(num_ctrl_qubits)
+        return decomposition
+
+
+class MCXSynthesisDefault(HighLevelSynthesisPlugin):
+    r"""The default synthesis plugin for a multi-controlled X gate.
+
+    This plugin name is :``mcx.default`` which can be used as the key on
+    an :class:`~.HLSConfig` object to use this method with :class:`~.HighLevelSynthesis`.
+
+
+    The plugin supports the following plugin-specific options:
+
+    * ``optimization_metric``: The optimization metric, indicating the property
+      of the output circuit (e.g., the 2-qubit gate count or the T-count) that should
+      be minimized. See :class:`.OptimizationMetric`.
+    * ``num_clean_ancillas``: The number of clean ancillary qubits available.
+    * ``num_dirty_ancillas``: The number of dirty ancillary qubits available.
+
+    """
+
+    def run(self, high_level_object, coupling_map=None, target=None, qubits=None, **options):
+        """Run synthesis for the given MCX gate."""
+
+        if not isinstance(high_level_object, (MCXGate, C3XGate, C4XGate)):
+            # Unfortunately we occasionally have custom instructions called "mcx"
+            # which get wrongly caught by the plugin interface. A simple solution is
+            # to return None in this case, since HLS would proceed to examine
+            # their definition as it should.
+            return None
+
+        metric = options.get("optimization_metric", OptimizationMetric.COUNT_2Q)
+        if metric == OptimizationMetric.COUNT_T:
+            # The order is optimized towards Clifford+T -friendly synthesis methods.
+            # In particular, we run 2DirtyKG24 and 1DirtyKG24 before NCleanM15 and NDirtyI15.
+            methods = [
+                MCXSynthesis2CleanKG24,
+                MCXSynthesis1CleanKG24,
+                MCXSynthesis2DirtyKG24,
+                MCXSynthesis1DirtyKG24,
+                MCXSynthesis1CleanB95,
+                MCXSynthesisNCleanM15,
+                MCXSynthesisNDirtyI15,
+                (
+                    MCXSynthesisNoAuxV24
+                    if high_level_object.num_ctrl_qubits <= 5
+                    else MCXSynthesisNoAuxHP24
+                ),
+            ]
+        else:
+            # The order is optimized towards CX-count -friendly synthesis methods.
+            # In particular, we run NCleanM15 and NDirtyI15 before 2DirtyKG24 and 1DirtyKG24.
+            methods = [
+                MCXSynthesis2CleanKG24,
+                MCXSynthesis1CleanKG24,
+                MCXSynthesisNCleanM15,
+                MCXSynthesisNDirtyI15,
+                MCXSynthesis2DirtyKG24,
+                MCXSynthesis1DirtyKG24,
+                MCXSynthesis1CleanB95,
+                (
+                    MCXSynthesisNoAuxV24
+                    if high_level_object.num_ctrl_qubits <= 5
+                    else MCXSynthesisNoAuxHP24
+                ),
+            ]
+
+        for method in methods:
+            if (
+                decomposition := method().run(
+                    high_level_object, coupling_map, target, qubits, **options
+                )
+            ) is not None:
+                return decomposition
+
+        return None
+
+
+class MCMTSynthesisDefault(HighLevelSynthesisPlugin):
+    """A default decomposition for MCMT gates."""
+
+    def run(self, high_level_object, coupling_map=None, target=None, qubits=None, **options):
+        if not isinstance(high_level_object, MCMTGate):
+            return None
 
         for synthesis_method in [
-            MCXSynthesis2CleanKG24,
-            MCXSynthesis1CleanKG24,
-            MCXSynthesisNCleanM15,
-            MCXSynthesisNDirtyI15,
-            MCXSynthesis2DirtyKG24,
-            MCXSynthesis1DirtyKG24,
-            MCXSynthesis1CleanB95,
+            MCMTSynthesisXGate,
+            MCMTSynthesisVChain,
         ]:
             if (
                 decomposition := synthesis_method().run(
@@ -1450,27 +1570,6 @@ class MCXSynthesisDefault(HighLevelSynthesisPlugin):
             ) is not None:
                 return decomposition
 
-        # If no synthesis method was successful, fall back to the default
-        return MCXSynthesisNoAuxV24().run(
-            high_level_object, coupling_map, target, qubits, **options
-        )
-
-
-class MCMTSynthesisDefault(HighLevelSynthesisPlugin):
-    """A default decomposition for MCMT gates."""
-
-    def run(self, high_level_object, coupling_map=None, target=None, qubits=None, **options):
-        # first try to use the V-chain synthesis if enough auxiliary qubits are available
-        if not isinstance(high_level_object, MCMTGate):
-            return None
-
-        if (
-            decomposition := MCMTSynthesisVChain().run(
-                high_level_object, coupling_map, target, qubits, **options
-            )
-        ) is not None:
-            return decomposition
-
         return MCMTSynthesisNoAux().run(high_level_object, coupling_map, target, qubits, **options)
 
 
@@ -1522,6 +1621,22 @@ class MCMTSynthesisVChain(HighLevelSynthesisPlugin):
         )
 
 
+class MCMTSynthesisXGate(HighLevelSynthesisPlugin):
+    """A synthesis for ``MCMTGate`` with X gate as the base gate."""
+
+    def run(self, high_level_object, coupling_map=None, target=None, qubits=None, **options):
+        if not isinstance(high_level_object, MCMTGate):
+            return None
+
+        if not isinstance(high_level_object.base_gate, XGate):
+            return None  # this plugin only supports X gates
+
+        ctrl_state = options.get("ctrl_state", None)
+        return synth_mcmt_xgate(
+            high_level_object.num_ctrl_qubits, high_level_object.num_target_qubits, ctrl_state
+        )
+
+
 class IntComparatorSynthesisDefault(HighLevelSynthesisPlugin):
     """The default synthesis for ``IntegerComparatorGate``.
 
@@ -1569,12 +1684,13 @@ class ModularAdderSynthesisDefault(HighLevelSynthesisPlugin):
     This plugin name is:``ModularAdder.default`` which can be used as the key on
     an :class:`~.HLSConfig` object to use this method with :class:`~.HighLevelSynthesis`.
 
-    If at least one clean auxiliary qubit is available, the :class:`ModularAdderSynthesisC04`
-    is used, otherwise :class:`ModularAdderSynthesisD00`.
-
     The plugin supports the following plugin-specific options:
 
-    * ``num_clean_ancillas``: The number of clean auxiliary qubits available.
+    * ``optimization_metric``: The optimization metric, indicating the property
+      of the output circuit (e.g., the 2-qubit gate count or the T-count) that should
+      be minimized. See :class:`.OptimizationMetric`.
+    * ``num_clean_ancillas``: The number of clean ancillary qubits available.
+    * ``num_dirty_ancillas``: The number of dirty ancillary qubits available.
 
     """
 
@@ -1582,26 +1698,45 @@ class ModularAdderSynthesisDefault(HighLevelSynthesisPlugin):
         if not isinstance(high_level_object, ModularAdderGate):
             return None
 
-        # For up to 5 qubits, the QFT-based adder is best
-        if high_level_object.num_state_qubits <= 5:
-            decomposition = ModularAdderSynthesisD00().run(
-                high_level_object, coupling_map, target, qubits, **options
-            )
-            if decomposition is not None:
+        metric = options.get("optimization_metric", OptimizationMetric.COUNT_2Q)
+        if metric == OptimizationMetric.COUNT_2Q:
+            # The order is optimized towards CX -friendly synthesis methods.
+            methods = []
+            if 2 <= high_level_object.num_state_qubits <= 4:
+                methods.append(ModularAdderSynthesisD00)
+            methods.append(ModularAdderSynthesisV17)
+        else:
+            # The order is optimized towards Clifford+T -friendly synthesis methods.
+            methods = [ModularAdderSynthesisV17]
+
+        for method in methods:
+            if (
+                decomposition := method().run(
+                    high_level_object, coupling_map, target, qubits, **options
+                )
+            ) is not None:
                 return decomposition
 
-        # Otherwise, the following decomposition is best (if there are enough ancillas)
-        if (
-            decomposition := ModularAdderSynthesisC04().run(
-                high_level_object, coupling_map, target, qubits, **options
-            )
-        ) is not None:
-            return decomposition
+        return None
 
-        # Otherwise, use the QFT-adder again
-        return ModularAdderSynthesisD00().run(
-            high_level_object, coupling_map, target, qubits, **options
-        )
+
+class ModularAdderSynthesisV17(HighLevelSynthesisPlugin):
+    r"""A modular adder (modulo :math:`2^n`) without any ancillary qubits.
+
+    The plugin name is :``ModularAdder.v17`` which can be used as the key on
+    an :class:`~.HLSConfig` object to use this method with :class:`~.HighLevelSynthesis`.
+
+    This plugin requires no auxiliary qubits.
+
+    """
+
+    def run(self, high_level_object, coupling_map=None, target=None, qubits=None, **options):
+        if not isinstance(high_level_object, ModularAdderGate):
+            return None
+
+        num_state_qubits = high_level_object.num_state_qubits
+
+        return adder_modular_v17(num_state_qubits)
 
 
 class ModularAdderSynthesisC04(HighLevelSynthesisPlugin):
@@ -1903,6 +2038,23 @@ class MultiplierSynthesisR17(HighLevelSynthesisPlugin):
         )
 
 
+class MultiplierSynthesisDefault(HighLevelSynthesisPlugin):
+    """THe default multiplier plugin.
+
+    This plugin name is:``Multiplier.default`` which can be used as the key on
+    an :class:`~.HLSConfig` object to use this method with :class:`~.HighLevelSynthesis`.
+    """
+
+    def run(self, high_level_object, coupling_map=None, target=None, qubits=None, **options):
+        if not isinstance(high_level_object, MultiplierGate):
+            return None
+
+        # The H18 algorithm is better both for CX-count and T-count
+        return multiplier_cumulative_h18(
+            high_level_object.num_state_qubits, high_level_object.num_result_qubits
+        )
+
+
 class PauliEvolutionSynthesisDefault(HighLevelSynthesisPlugin):
     """Synthesize a :class:`.PauliEvolutionGate` using the default synthesis algorithm.
 
@@ -2093,6 +2245,7 @@ class AnnotatedSynthesisDefault(HighLevelSynthesisPlugin):
             use_physical_indices=data.use_physical_indices,
             min_qubits=0,
             unroll_definitions=data.unroll_definitions,
+            optimize_clifford_t=data.optimize_clifford_t,
         )
 
         num_ctrl = sum(mod.num_ctrl_qubits for mod in modifiers if isinstance(mod, ControlModifier))
