@@ -74,6 +74,7 @@ def circuit_drawer(
     cregbundle: bool | None = None,
     wire_order: list[int] | None = None,
     expr_len: int = 30,
+    measure_arrows: bool | None = None,
 ):
     r"""Draw the quantum circuit. Use the output parameter to choose the drawing format:
 
@@ -171,6 +172,11 @@ def circuit_drawer(
         expr_len: The number of characters to display if an :class:`~.expr.Expr`
             is used for the condition in a :class:`.ControlFlowOp`. If this number is exceeded,
             the string will be truncated at that number and '...' added to the end.
+        measure_arrows: If True, draw an arrow from each measure box down the the classical bit
+            or register where the measure value is placed. If False, do not draw arrow, but
+            instead place the name of the bit or register in the measure box.
+            Default is ``True`` unless the user config file (usually ``~/.qiskit/settings.conf``)
+            has an alternative value set. For example, ``circuit_measure_arrows = False``.
 
     Returns:
         :class:`.TextDrawing` or :class:`matplotlib.figure` or :class:`PIL.Image` or
@@ -208,6 +214,7 @@ def circuit_drawer(
     default_output = "text"
     default_reverse_bits = False
     default_idle_wires = config.get("circuit_idle_wires", "auto")
+    default_measure_arrows = config.get("circuit_measure_arrows", True)
     if config:
         default_output = config.get("circuit_drawer", "text")
         if default_output == "auto":
@@ -231,6 +238,9 @@ def circuit_drawer(
         else:
             raise VisualizationError(f"Parameter idle_wires={idle_wires} unrecognized.")
 
+    if measure_arrows is None:
+        measure_arrows = default_measure_arrows
+
     if wire_order is not None and reverse_bits:
         raise VisualizationError(
             "The wire_order option cannot be set when the reverse_bits option is True."
@@ -304,6 +314,7 @@ def circuit_drawer(
             cregbundle=cregbundle,
             wire_order=complete_wire_order,
             expr_len=expr_len,
+            measure_arrows=measure_arrows,
         )
     elif output == "latex":
         image = _latex_circuit_drawer(
@@ -352,6 +363,7 @@ def circuit_drawer(
             cregbundle=cregbundle,
             wire_order=complete_wire_order,
             expr_len=expr_len,
+            measure_arrows=measure_arrows,
         )
     else:
         raise VisualizationError(
@@ -383,6 +395,7 @@ def _text_circuit_drawer(
     encoding=None,
     wire_order=None,
     expr_len=30,
+    measure_arrows=True,
 ):
     """Draws a circuit using ascii art.
 
@@ -415,6 +428,9 @@ def _text_circuit_drawer(
         expr_len (int): Optional. The number of characters to display if an :class:`~.expr.Expr`
             is used for the condition in a :class:`.ControlFlowOp`. If this number is exceeded,
             the string will be truncated at that number and '...' added to the end.
+        measure_arrows: If True, draw an arrow from each measure box down the the classical bit
+            or register where the measure value is placed. If False, do not draw arrow, but
+            instead place the name of the bit or register in the measure box.
 
     Returns:
         TextDrawing: An instance that, when printed, draws the circuit in ascii art.
@@ -428,6 +444,7 @@ def _text_circuit_drawer(
         justify=justify,
         idle_wires=idle_wires,
         wire_order=wire_order,
+        measure_arrows=measure_arrows,
     )
     text_drawing = _text.TextDrawing(
         qubits,
@@ -440,6 +457,7 @@ def _text_circuit_drawer(
         encoding=encoding,
         with_layout=with_layout,
         expr_len=expr_len,
+        measure_arrows=measure_arrows,
     )
     text_drawing.plotbarriers = plot_barriers
     text_drawing.line_length = fold
@@ -666,6 +684,7 @@ def _matplotlib_circuit_drawer(
     cregbundle=None,
     wire_order=None,
     expr_len=30,
+    measure_arrows=None,
 ):
     """Draw a quantum circuit based on matplotlib.
     If `%matplotlib inline` is invoked in a Jupyter notebook, it visualizes a circuit inline.
@@ -700,6 +719,9 @@ def _matplotlib_circuit_drawer(
         expr_len (int): Optional. The number of characters to display if an :class:`~.expr.Expr`
             is used for the condition in a :class:`.ControlFlowOp`. If this number is exceeded,
             the string will be truncated at that number and '...' added to the end.
+        measure_arrows: If True, draw an arrow from each measure box down the the classical bit
+            or register where the measure value is placed. If False, do not draw arrow, but
+            instead place the name of the bit or register in the measure box.
 
     Returns:
         matplotlib.figure: a matplotlib figure object for the circuit diagram
@@ -712,6 +734,7 @@ def _matplotlib_circuit_drawer(
         justify=justify,
         idle_wires=idle_wires,
         wire_order=wire_order,
+        measure_arrows=measure_arrows,
     )
     if fold is None:
         fold = 25
@@ -731,5 +754,6 @@ def _matplotlib_circuit_drawer(
         cregbundle=cregbundle,
         with_layout=with_layout,
         expr_len=expr_len,
+        measure_arrows=measure_arrows,
     )
     return qcd.draw(filename)
