@@ -21,13 +21,14 @@ from ddt import ddt, data, unpack
 
 from qiskit import transpile
 from qiskit.circuit import QuantumCircuit, Parameter
-from qiskit.circuit.library import PauliEvolutionGate, HamiltonianGate, PhaseGate
+from qiskit.circuit.library import PauliEvolutionGate, HamiltonianGate, PhaseGate, RZGate
 from qiskit.synthesis import LieTrotter, SuzukiTrotter, MatrixExponential, QDrift
 from qiskit.synthesis.evolution.product_formula import reorder_paulis
 from qiskit.converters import circuit_to_dag
 from qiskit.quantum_info import Operator, SparsePauliOp, Pauli, Statevector, SparseObservable
 from qiskit.transpiler.passes import HLSConfig, HighLevelSynthesis
 from qiskit.utils import optionals
+from qiskit.circuit._utils import _compute_control_matrix
 from test import QiskitTestCase, combine  # pylint: disable=wrong-import-order
 
 X = SparsePauliOp("X")
@@ -59,16 +60,37 @@ class TestEvolutionGate(QiskitTestCase):
         self.assertTrue(Operator(gate).equiv(exact_gate))
 
     def test_matrix_decomposition(self):
-        """Test the default decomposition."""
+        """Test the matrix decomposition."""
         op = (X ^ X ^ X) + (Y ^ Y ^ Y) + (Z ^ Z ^ Z)
         time = 0.123
 
         matrix = op.to_matrix()
         evolved = scipy.linalg.expm(-1j * time * matrix)
 
-        evo_gate = PauliEvolutionGate(op, time, synthesis=MatrixExponential())
+        with self.subTest(msg="test MatrixExponential"):
+            evo_gate = PauliEvolutionGate(op, time, synthesis=MatrixExponential())
+            self.assertTrue(Operator(evo_gate).equiv(evolved))
 
-        self.assertTrue(Operator(evo_gate).equiv(evolved))
+        with self.subTest(msg="test to_matrix"):
+            evo_gate = PauliEvolutionGate(op, time)
+            self.assertTrue(np.allclose(evolved, evo_gate.to_matrix()))
+
+    def test_to_matrix_commuting_blocks(self):
+        """Test to_matrix if the evolution is specified with commuting blocks."""
+        block_1q = (X ^ I) + (I ^ X)
+        block_2q = (X ^ X) + (Y ^ Y) + (Z ^ Z)
+
+        evo = PauliEvolutionGate([block_1q, block_2q])
+        matrix = evo.to_matrix()
+
+        expected = PauliEvolutionGate(block_1q + block_2q).to_matrix()
+        self.assertTrue(np.allclose(expected, matrix))
+
+    def test_to_matrix_invalid_time(self):
+        """Test calling to_matrix with a non-numeric time fails."""
+        evo = PauliEvolutionGate(Z, time=Parameter("time"))
+        with self.assertRaises(ValueError):
+            _ = evo.to_matrix()
 
     def test_reorder_paulis_invariant(self):
         """
@@ -430,6 +452,50 @@ class TestEvolutionGate(QiskitTestCase):
 
         self.assertTrue(Operator(circuit).equiv(np.identity(2**circuit.num_qubits)))
 
+    def test_inverse_to_matrix(self):
+        """Test inverse().to_matrix()."""
+        op = X + Y
+        time = 0.512
+        exact = scipy.linalg.expm(-1j * time * op.to_matrix())
+        exact_inv = np.transpose(np.conjugate(exact))
+
+        evo = PauliEvolutionGate(op, time)
+        evo_inv = evo.inverse()
+
+        self.assertTrue(np.allclose(exact, evo.to_matrix()))
+        self.assertTrue(np.allclose(exact_inv, evo_inv.to_matrix()))
+
+    def test_power(self):
+        """Test calling the power method."""
+        power = 31
+        dt = 0.1
+        op = (X ^ X) + (Z ^ Z)  # pick commuting operator to check matrices
+
+        evo = PauliEvolutionGate(op, time=dt)
+        evo_pow = evo.power(power)
+
+        expect = PauliEvolutionGate(op, time=power * dt)
+
+        with self.subTest("check efficient power"):
+            self.assertEqual(expect, evo_pow)
+
+        matrix = scipy.linalg.expm(-1j * dt * power * op.to_matrix())
+        with self.subTest("check unitary"):
+            self.assertTrue(np.allclose(matrix, Operator(evo_pow).data))
+
+    def test_control_grouped(self):
+        """Test the control method on grouped operators."""
+        block_1q = (I ^ X) + (X ^ I)
+        block_2q = (Z ^ Z) + (Y ^ Y) + (X ^ X)
+
+        evo = PauliEvolutionGate([block_1q, block_2q], time=1, synthesis=LieTrotter())
+        controlled = evo.control(2, ctrl_state="01")
+
+        summed = PauliEvolutionGate(block_1q + block_2q, time=1, synthesis=LieTrotter())
+        reference = summed.control(2, ctrl_state="01")
+
+        self.assertEqual(reference, controlled)
+
     def test_labels_and_name(self):
         """Test the name and labels are correct."""
         operators = [
@@ -540,7 +606,7 @@ class TestEvolutionGate(QiskitTestCase):
         evo = PauliEvolutionGate(Z, time=time)
 
         angle = evo.definition.data[0].operation.params[0]
-        expected = (2.0 * time).sympify()
+        expected = (2 * time).sympify()
         self.assertEqual(expected, angle.sympify())
 
     def test_zero(self):
@@ -726,6 +792,42 @@ class TestEvolutionGate(QiskitTestCase):
         # we should also be less (or equal) to this
         self.assertLessEqual(cx_count, num_cx)
 
+    @data("110", 6)
+    def test_ctrl_state(self, ctrl_state):
+        """Test controlled evolution gate with a control state."""
+        obs = SparseObservable("ZZ")
+        evo = PauliEvolutionGate(obs)
+        controlled = evo.control(num_ctrl_qubits=3, ctrl_state=ctrl_state)
+        qc = controlled.definition
+
+        reference = QuantumCircuit(*qc.qregs)
+        reference.cx(4, 3)
+        reference.append(RZGate(2).control(3, ctrl_state="011"), [2, 1, 0, 3])
+        reference.cx(4, 3)
+        with self.subTest("check decomp"):
+            self.assertEqual(reference, qc)
+
+        zz_mat = np.diag([1, -1, -1, 1])
+        rzz_mat = scipy.linalg.expm(-1j * zz_mat)
+        ctrl_mat = _compute_control_matrix(rzz_mat, 3, ctrl_state)
+        with self.subTest("check correctness"):
+            self.assertTrue(np.allclose(ctrl_mat, Operator(qc).data))
+
+    def test_raises_on_empty_list(self):
+        """Test that an error gets raised when a Pauli evolution gate is created from an empty list."""
+        with self.assertRaises(ValueError):
+            PauliEvolutionGate([], time=1)
+
+    def test_raises_on_list_with_different_num_qubits(self):
+        """Test that an error gets raised when a Pauli evolution gate is created from a list,
+        where not all of the operators have the same number of qubits.
+        """
+
+        with self.assertRaises(ValueError):
+            pauli = Pauli("XYZ")  # 3 qubits
+            op = SparsePauliOp(["XYIZ"], [1])  # 4 qubits
+            PauliEvolutionGate([pauli, op], time=1)
+
 
 def exact_atomic_evolution(circuit, pauli, time):
     """An exact atomic evolution for Suzuki-Trotter.
