@@ -782,6 +782,21 @@ class TestOptimize1qGatesDecomposition(QiskitTestCase):
         expected.u(0, 0, np.pi / 2, [0])
         self.assertEqual(result, expected)
 
+    def test_target_fixed_angle_1q_gate(self):
+        """Test that a fixed angle gate is not used for decomposition."""
+        target = Target(num_qubits=1)
+        target.add_instruction(UGate(3.14, 0, -3.14), {(0,): None})
+        target.add_instruction(SXGate(), {(0,): None})
+        target.add_instruction(RZGate(Parameter("t")), {(0,): None})
+
+        opt_pass = Optimize1qGatesDecomposition(target=target)
+        qc = QuantumCircuit(1)
+        qc.u(1, 2, 3, 0)
+        res = opt_pass(qc)
+        self.assertNotIn("u", res.count_ops())
+        self.assertIn("rz", res.count_ops())
+        self.assertIn("sx", res.count_ops())
+
 
 if __name__ == "__main__":
     unittest.main()
