@@ -12,23 +12,25 @@
 
 """Test the VF2Layout pass"""
 
+import ddt
 import rustworkx
 
 from qiskit import QuantumRegister, QuantumCircuit
 from qiskit.circuit import ControlFlowOp
 from qiskit.circuit.library import CXGate, XGate
-from qiskit.transpiler import Layout, TranspilerError
+from qiskit.transpiler import Layout, TranspilerError, PassManager, passes
 from qiskit.transpiler.passes.layout.vf2_post_layout import VF2PostLayout, VF2PostLayoutStopReason
 from qiskit.converters import circuit_to_dag
 from qiskit.providers.fake_provider import GenericBackendV2
 from qiskit.circuit import Qubit
 from qiskit.compiler.transpiler import transpile
 from qiskit.transpiler.target import Target, InstructionProperties
-from test import QiskitTestCase  # pylint: disable=wrong-import-order
+from test import QiskitTestCase, combine  # pylint: disable=wrong-import-order
 
 from ..legacy_cmaps import LIMA_CMAP, YORKTOWN_CMAP, BOGOTA_CMAP
 
 
+@ddt.ddt
 class TestVF2PostLayout(QiskitTestCase):
     """Tests the VF2Layout pass"""
 
@@ -324,6 +326,59 @@ class TestVF2PostLayout(QiskitTestCase):
         vf2_pass.run(dag)
         self.assertLayoutV2(dag, target_last_qubits_best, vf2_pass.property_set)
 
+    @combine(
+        seed=(-1, 12),  # This hits both the "seeded" and "unseeded" paths.
+        strict_direction=(True, False),
+    )
+    def test_complete_layout_with_idle_qubits(self, seed, strict_direction):
+        """Test that completely idle qubits are included in the resulting layout."""
+        qc = QuantumCircuit(3)
+        qc.cx(0, 1)
+        # We need to ensure that VF2Post actually triggers a remapping.
+        target = Target(3)
+        target.add_instruction(
+            CXGate(),
+            properties={
+                (0, 1): InstructionProperties(error=1e-1),
+                (1, 0): InstructionProperties(error=1e-1),
+                (2, 1): InstructionProperties(error=1e-8),
+            },
+        )
+        property_set = {}
+        pass_ = VF2PostLayout(target=target, seed=seed, strict_direction=strict_direction)
+        pass_(qc, property_set=property_set)
+        unallocated = {
+            i for i, bit in enumerate(qc.qubits) if bit not in property_set["post_layout"]
+        }
+        self.assertEqual(unallocated, set())
+
+    @ddt.data(-1, 12)
+    def test_complete_layout_with_one_uncoupled_qubit(self, seed):
+        """Test that a single uncoupled qubit is handled correctly.
+
+        Regression test of https://github.com/Qiskit/qiskit/issues/14997."""
+        qc = QuantumCircuit(3)
+        qc.x(0)
+        qc.cx(1, 2)
+
+        bad = InstructionProperties(error=1e-1)
+        good = InstructionProperties(error=1e-5)
+
+        target = Target()
+        target.add_instruction(XGate(), {(0,): bad, (1,): good, (2,): good})
+        target.add_instruction(CXGate(), {(0, 1): good, (1, 2): bad})
+
+        pm = PassManager(
+            [
+                passes.TrivialLayout(target),
+                passes.ApplyLayout(),
+                passes.VF2PostLayout(target, seed=seed, strict_direction=True),
+                passes.ApplyLayout(),
+            ]
+        )
+        out = pm.run(qc)
+        self.assertEqual(out.layout.final_index_layout(), [2, 0, 1])
+
 
 class TestVF2PostLayoutScoring(QiskitTestCase):
     """Test scoring heuristic function for VF2PostLayout."""
